% -*- coding:utf-8 mode:trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: core-syntax.pl,v $
%%  $Revision: 282 $
%%      $Date: 2008-01-12 11:59:30 +0100 (Sa, 12 Jan 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: All the syntactic constrints that belong to
%%             Universal Grammar.
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.10.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.


% für
% Die Aufsätze wurden ihm zu lesen erlaubt.
%
% 1 min 30.740 sec CPU time.        1 solutions; 236 passive edges
% statt
% 2 min 52.210 sec CPU time.        1 solutions; 334 passive edges

% Diese Formulierung arbeitet mit delays. Die Disjunktion kommt erst am Ende
% der Liste ins Spiel, weshalb das ganze effizienter ist. Diese Formulierung
% ist jedoch nicht allgemeingültig, da der Fall np_str,np_lex explizit aufgeführt
% wird, das entspricht nicht genau der Formulierung: "Erster Nominativ"
% 05.06.2006

% Geht durch eine Liste und gibt dem letzten Element die
% Person und Numerus-Merkmale, wenn die Liste mit einer NP_str
% endet. Ansonsten muß das Verb dritte Perosn sg sein.

% Dieses Constraint wird gebraucht, da Modalverben
% nicht wissen, ob sie ein Subjekt haben.


% Subjekt-Verb-Kongruenz
%
% []                kommt nicht vor
% [V[ppp]           wird gearbeitet
% [np_lex]          dürsten
% [np_str]          schlafen
% [np_lex, np_str]  helfen
% [cp]              stimmen
% [np_lex, cp]      auffallen


% there is a list of arguments
% the last one may be a subject. If it is a subject, i.e. np_str_nom
% it has to agree with the verb.
% If the last element is not a noun or has lexical case, the verb
% has to be third sg.

%subj_verb_agreement(_,_) if !,true.  % for debugging


fun subj_verb_agreement(-,+).
subj_verb_agreement([H|T],Index) if
  when( T=(e_list;ne_list)
      , undelayed_subj_verb_agreement([H|T],Index)
      ).

% For debugging
%undelayed_subj_verb_agreement([El],Index) if prolog((write(El),nl,write(Index),nl)),fail.

% Das sind alle Fälle, in denen T=e_list ist.
% This formulation of the clause is due to a bizarre Trale Bug. If one uses the np_str(Index)
% instead, Trale fails for "Der Aufsatz gelesen wurde" with PVP fronting, but succeds on the same sentence
% with verb final pattern. St. Mü. 18.05.2007 Check behaviour with new Trale.
%


% According to Kamiri2005a:97 agreement is optional if the subject is inanimate in Persian.
% 
% The number is specified, since otherwise one would geeet spurious ambiguities in the singular,
% since here the normal agrreement is iinndistinguishable from default agreement.
undelayed_subj_verb_agreement([(@np_str,
                                @arg(loc:cont:ind:(num:pl,
                                                   sort:non_agentive)))],     (per:third,num:sg))   if true.




undelayed_subj_verb_agreement([(@np_str,
                                @arg(loc:cont:ind:Index))],      Index)   if true.
undelayed_subj_verb_agreement([@np_lex],             (per:third,num:sg))  if true.
undelayed_subj_verb_agreement([@no_noun],            (per:third,num:sg))  if true.

% Also muß T jetzt ne_list sein, d.h. die Liste hat mindestens zwei Elemente
undelayed_subj_verb_agreement(List,Index)                          if subj_verb_agreement2(List,Index).

% Wir warten, bis klar ist, wie der Listenrest aussieht.
fun subj_verb_agreement2(-,+,+).
subj_verb_agreement2([H,H2|T],Index) if
  when( T=(e_list;ne_list)
      , undelayed_subj_verb_agreement2([H,H2|T],Index)
      ).

% Wenn es genau zwei Elemente gibt, kann der Fall eintreten, daß
% das erste Element lexikalisch und das zweite strukturell ist.
% Wenn das so ist, gibt es Kongruenz, und wir sind fertig.
undelayed_subj_verb_agreement2([@np_str(Index),@np_lex], Index) if !,true.

% Wenn das nicht der Fall ist, verkürzen wir die Liste um ein Argument
% und testen erneut alles ab.
undelayed_subj_verb_agreement2(tl:T1,                   Index)   if subj_verb_agreement(T1,Index).

