% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: common.pl,v $
%%  $Revision: 223 $
%%      $Date: 2007-09-12 19:27:44 +0200 (Wed, 12 Sep 2007) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Eine kleine Spielzeuggrammatik für die Lehre
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





% Hack to reset the timer.

:- statistics(runtime,_).


compile_grammar_hook :-
         inform_user_about_top_level,
         print_examples,
%         statistics(walltime,[_,T]),
         statistics(runtime,[_,T]),
         msec2time(T,Time),
         number_of(lex(_,_),NumberOfWords),
         format(user_error,"~nGrammatik geladen: ~w~n~w Lexikoneinheiten~n~n",[Time,NumberOfWords]),
%         r,   % diese Zeile auskommentieren, falls die Stämme zum Debuggen im Lexikon bleiben sollen
         ( current_predicate(grammar_hook,grammar_hook) -> call_all(grammar_hook)
         ; true).
         

% Interferes with TSDB stuff

%          % If we are at the Prolog top level, ...
%          ( trale_mode(prolog) ->
%              go

%          ;   true
%          ).



t :- testt(all).


% Wie in trale_patches.pl, aber mit Statistik-Ausgaben.

% :- abolish(compile_gram/1).

% compile_gram(File) :-
%   clear_feature_output,   % initialize feature_ordering.pl
%   abolish_preds,
%   reconsult(File),
%   statistics(walltime,_),       % rücksetzen des Timers
%   compile_gram,
%   process_feature_output,       % compile info in feature_ordering.pl
%   statistics(walltime,[_,T]),
%   msec2time(T,Time),
%   number_of(lex(_,_),NumberOfWords),
%   format(user_error,"~nGrammatik geladen: ~w~n~w Lexikoneinheiten~n~n",[Time,NumberOfWords]),
%   r, % diese Zeile auskommentieren, falls die Stämme zum Debuggen im Lexikon bleiben sollen
%   number_of(lex(_,_),NumberOfRemainingWords),
%   format(user_error,"~w verbleibende Lexikoneinheiten~n~n",[NumberOfRemainingWords]),
%   compile_grammar_hook.



:- [trale_home('tsdb/retract_lex_desc')].


% % Dieses Prädikat entfernt die Stämme aus dem Lexikon.
% % In der syntaktischen Analyse braucht man die Stämme nicht,
% % die Grammatikregeln sind ohnehin so formuliert, daß Stämme
% % nicht als Töchter vorkommen können. Stämme für Nomina (Frau, Mann, Haus)
% % und Adjektive (klug, interessant) würden aber beim Parsen in die Parse-Tabelle
% % eingetragen und bei jedem Parse-Schritt wieder überprüft. Diese Arbeit kann
% % man sich sparen, wenn man die Stämme nach der Lexikonberechnung löscht.
r :-    format(user_error,"Lösche Stammeinträge ...~n",[]),
        open_null_stream(DevNull),
        prolog_flag(user_error,UserError,DevNull),
        retract_lex_desc(stem),
        prolog_flag(user_error,_,UserError),
        number_of(lex(_,_),NumberOfRemainingWords),
        format(user_error,"~w verbleibende Lexikoneinheiten~n~n",[NumberOfRemainingWords]).




:- abolish(morph/3).

% % Das ist eine Kopie des Codes aus ale.pl
% % + einer Zeile zur Ausgabe des aktuell erzeugten Wortes.
% % Das ist zum Debuggen und wegen der langen Lexikonkompilierungszeit sinnvoll.

morph(Morphs,WordIn,WordOut):-  % need to instantiate Word even if
  name(WordIn,CodesIn),         %  X becomes X - do we want this?
  make_char_list(CodesIn,CharsIn), 
  morph_chars(Morphs,CharsIn,CharsOut),
  make_char_list(CodesOut,CharsOut),
  name(WordOut,CodesOut),
  format(user_error,"~w~n",[WordOut]),
%  write(user_error,'.'),flush_output(user_error),
  true.





% If a description that should be used for generation is produced from a chart edge,
% which pathes of the input sign shall be considered?
gen_pathes([[synsem,loc,cat,head],[synsem,loc,cat,subcat],[synsem,loc,cont],[rels],[h_cons]]).



% Semantics Definition for Generation with the build-in generator
% ------------------------------------------------------------------------------

% semantics sem1.
% sem1(sem:Cont,Cont) if true.





% Semantics Definition for Generation with Aurelien's generator
% ------------------------------------------------------------------------------

%% PREREQUISITES
%%
%%  o These can be given in theory.pl:
%%          syntactic_object(<type being the mgsat of all syntactic objects>).
%%
%%          ind_path([path,to,index,of,the,mrs]).
%%
%%          cont_path([path,to,mrs]).
%%          liszt_path([path,to,list,of,EPs]).
%%          hcons_path([path,to,handle,constraints]).
%%
%%     Here is what I use. There are chances that it is similar to that:
%%
%%          syntactic_object(syntactic_object).
%%
%%          ind_path([synsem,loc,cont,ind]).
%%          cont_path([synsem,loc,cont]).
%%          liszt_path([synsem,loc,cont,rels]).
%%          hcons_path([synsem,loc,cont,h_cons]).
%%
%% LOADING
%%
%%  o either launch trale with otion:
%%          -e "compile('/path/to/generator/directory/generator.pl')"
%%    or call within trale:
%%          compile('/path/to/generator/directory/generator.pl').
%%
%% USE
%%
%%  o To parse a sentence and generate from its semantics: 
%%          p_and_g([sentence,as,list,of,words]).
%%
%%  o To parse a phrase of description Desc and generate from its semantics:
%%          p_and_g([sentence,as,list,of,words],Desc).
%%
%%  o To generate from a description Desc (representing the whole sign, not only the
%%    semantics), getting all the outputs in the list Words, and matching Desc to the
%%    description Root:
%%          g(Desc).
%%          g(Desc,Words).
%%          g_d(Desc,Words,Root).
%%          g_d(Desc,Root).


          syntactic_object(syntactic_object).

          ind_path([synsem,loc,cont,ind]).
          gtop_path([synsem,loc,cont,gtop]).
          cont_path([synsem,loc,cont]).
          liszt_path([rels]).
          hcons_path([h_cons]).

outscoped_feat(larg).
sc_arg_feat(harg).
scopable_description([@decl,@interrog,@ass_or_imp]).

quantifiers([udef_q,def_q,some_q,demonstrative_q,proper_q]).

%:- compile('generator.pl').



make_ :- ensure_loaded([trale_home(grammar_interface),
                       trale_home('tsdb/itsdb'),
                       trale_home('chart_display/chart_display'),
                       trale_home('chart_display/sic'),
%                       trale_home('chart_display/ale_redefinitions'),
                       trale_home('chart_display/print_mrs'),
                       trale_home('chart_display/print_mrs_utool'),
                       trale_home('chart_display/utool-interface'),
                  trale_home('tsdb/retract_lex_desc')    % retraction of stems
                  ,trale_home('tsdb/itsdb-err')              % capi-registration write-filed ...
                 ,trale_home('tsdb/trale-itsdb')
                      ,trale_home('debug_unify')
                      ,trale_home('test_suite_handling')
                      ,trale_home('toplevel')
                      ,test_items
                      ,tokenization
                 ]),
         discontinuous_edges,
         chart_display,
%         nofs,
%         german,
         notcl_warnings,
%         hrp,
         mrs,
         display_mrs,
         scope_mrs,
         debug_mrs,
         true.



%   The name of the ALE flag I referred to is ruleindexscope.  Its
% settings are:

% point [default]
% localtree
% localresolve

% The last of these does everything - goals in the rules, forall rule
% statements, etc.  But be careful - it hangs very easily.  localtree
% builds all of the categories, but does no procedural attachment
% resolution.  point just tests the two categories at the point of
% overlap in the indexed pair (a mother and first daughter). 

%:- ale_flag(ruleindexscope,_Old,localtree).
%:- ale_flag(ruleindexscope,_Old,localresolve).


% This can be used to disable loading of expansive lexical rules as for instance
% lexical rules for participles or depicitve adjectives.

% Since this has to be changed by hand, the default for a freshly loaded grammar
% is that everything is loaded.


%:- ale_flag(subtypecover,_Old,off).

% Example:
% :- skip_loading_for_debugging(off) -> ['clitic-lexrules'].

/*
:- dynamic skip_loading_for_debugging/1.
skip_loading_for_debugging(off).

skip_loading :-
        retractall(skip_loading_for_debugging(_)),
        assert(skip_loading_for_debugging(on)).


load_all :-
        retractall(skip_loading_for_debugging(_)),
        assert(skip_loading_for_debugging(off)).

*/



/*

% Webtrale

To use WebTrale with a grammar:


1. Start TRALE as server

- go to the grammar's directory
- start trale with the option -G
- compile the grammar (c.)
- load the file wtx.pl from WT_HOME/src/webtrale, e.g.

    :- ensure_loaded('/webtrale/src/webtrale/wtx.pl').

- call the predicate trale_start_server(Port), e.g.

    :- trale_server_start(9090).


2. Start WebTrale

- in another shell, execute

    $ java -jar WT_HOME/webtrale_lite.jar --trale-server-port=9090

- hint: add '--help' to the above line to see the options
  avaiilable to webtrale

> rÃ¢ should be râ
I see. To fix this, try exporting SP_CTYPE=utf-8 in the shell that
starts WebTrale, before executing 'java -jar webtrale...'.


*/


webtrale(X) :- ensure_loaded('/home/stefan/Prolog/Trale/Code/webtrale/src/webtrale/wtx.pl'),nochart_display,
trale_server_start(X).

% java -jar /home/stefan/Prolog/Trale/Code/webtrale/webtrale_lite.jar --trale-server-port=9090 --allow-remote-connections --no-auth --max-words=15 --trale-server-host=hpsg.fu-berlin.de --trale-server-port=7788 --multiuser
