% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: lexrules.pl,v $
%%  $Revision: 1.4 $
%%      $Date: 2005/03/03 15:11:24 $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Eine kleine Spielzeuggrammatik für die Lehre
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile '--->'/2.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile (rule)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.
:- multifile (if)/2.
:- discontiguous (if)/2.
        
% Coordination of verbal and non-verbal constituents



% Hm, the original motivation for CAT was that we are able to share it among conjuncts.

conjunction(Reln) :=
  (coord_conjunction_word,
      @overt_le,
      @non_slashed_le,
      @non_idiomatic_le,
      @non_rel_le,
      @non_que_le,
      @non_relativizer_le,
      @relational_arg0_le,
      @ltop_lbl_le,  
      synsem:loc:(cat:(head:(coord,
                             spec:(loc:(cat:(head:Head,
                                             %spr:Spr,
                                             marked_:Marked),
                                        cont:(ind:Ind1,
                                              ltop:LHandle,
                                              gtop:GTop)),
                                   nonloc:Nonloc,
                                   trace:minus)),
                       spr:[],
                       subcat:[(@argument,
                                arg:(loc:(cat:(head:Head,
                                               %spr:Spr,
                                               marked_:Marked), 
                                          cont:(ind:Ind2,
                                                ltop:RHandle,
                                                gtop:GTop)),
                                     nonloc:Nonloc,
                                     trace:minus))]),
                  cont:(ind:Ind0,
                        gtop:GTop)), % a complex event or a group index
      rels:coord_sem(Reln,Ind0,Ind1,Ind2,LHandle,RHandle,GTop,HCons),
      h_cons:HCons).




fun coord_sem(+,+,+,+,+,+,+,-,+).
%coord_sem(Pred,Ind1,Ind2,LHandle,RHandle,GTop,Rels,HCons)

coord_sem(Pred,Ind0,Ind1,Ind2,LHandle,RHandle,GTop,Rels,HCons) if
  when((Ind1=(index;event),
        Ind2=(index;event))
      ,undelayed_coord_sem(Pred,Ind0,Ind1,Ind2,LHandle,RHandle,GTop,Rels,HCons)).


% What about scope of the quantifiers in the coordinated NP?
%
% Alle Studenten lesen ein Buch und eine Zeitung.
%
% exists x buch(x) exists y Zeitung(y) alle studenten lesen x+y.
%
% exists x buch(x) alle studenten exists y Zeitung(y) lesen x+y.

% The nominal case with `and' we get a plural Index
undelayed_coord_sem(Pred,Ind0,Ind1,(ref, % this was index, but Persian ahs a type event_index, which is both due to the process nouns.
                                    Ind2),_LHandle,_RHandle,
          GTop,
         [(Pred,
           and_n_rel,
           lbl:ConjHandle,
           arg0:(Ind0,
                 per:third,
                 num:pl,
                 sort:SortM),  % The sort of the group is the generalization of the sorts of the members of the group.
           arg1:(Ind1,
                 sort:Sort1),
           arg2:(Ind2,
                 sort:Sort2)
% In the case of nouns, it is not the handle of the noun that is plugged in here.          
%           ,l_handle:LHandle,
%           r_handle:RHandle
          ),
          (udef_q,        
           lbl:QHandle,
           arg0:Ind0,
           rstr:Restr)],
         [(qeq,
           harg:GTop,
           larg:QHandle),
          (qeq,
           harg:Restr,
           larg:ConjHandle)]) if prolog(mspec_sup_type(Sort1,Sort2,SortM)).

% The nominal case with `or' we get agreement with the last conjunct (at least in German ...)
undelayed_coord_sem(Pred,Ind0,Ind1,(ref, % this was index, but Persian ahs a type event_index, which is both due to the process nouns.
                          Ind2),_LHandle,_RHandle,
          GTop,
         [(Pred,
           or_n_rel,
           lbl:ConjHandle,
           arg0:(Ind0,
                 per:Per,
                 num:Num),
           arg1:Ind1,
           arg2:(Ind2,
                 per:Per,
                 num:Num)
% In the case of nouns, it is not the handle of the noun that is plugged in here.          
%           l_handle:LHandle,
%           r_handle:RHandle
          ),
          (udef_q,        
           lbl:QHandle,
           arg0:Ind0,
           rstr:Restr)],
         [(qeq,
           harg:GTop,
           larg:QHandle),
          (qeq,
           harg:Restr,
           larg:ConjHandle)]) if true.


% The verbal case:
undelayed_coord_sem(Pred,Ind0,Ind1,(event,
                                    Ind2),LHandle,RHandle,
          _GTop,
         [(Pred,
           verbal_conjunction_rel,  % TRALE should be able to infer this ...
                                    % However, the new TRALE just fails on constructing the conjunction lexical item for
                                    % coordination of verbal constituents, when this type is not specified.
                                    % St. Mü. 18.06.2010 TRALE bug
           arg0:(Ind0,
                 event),
           arg1:Ind1,
           arg2:Ind2,
           l_handle:LHandle,
           r_handle:RHandle)],
         []) if true.

% without this constraint a conjunctionn could be combined with
% a combination of an extraction trace and a verb trace:
%
% und [_e _v]
%
% This element is not a trace itself, but of course it is unwanted as an
% argument of the conjunction.

(headed_phrase,
 synsem:loc:cat:head:coord) *> non_head_dtrs:hd:phon:ne_list.

coord_phrase *>
  (non_head_dtrs:[(phon:ne_list,
                   synsem:trace:minus),
                  synsem:trace:minus]).

x_conj_y_coord_phrase :=
  (coord_phrase,
   synsem:(loc:(cat:(head:Head,
                     xarg:accumulated_args(XArg1,XArg2),
                     spr:unrealized_args(Spr1,Spr2),
                     subcat:(% for reasons I do not understand this does not work.
                             % The constraints seem to influence each other in strange ways.
                             % With the result that both the new and the old TRALE hang (at different places).
                             % TRALE bug         18.06.2010 St. Mü.
                             %unrealized_args(Subcat1),
                             %unrealized_args(Subcat2)%,

                             unrealized_args(Subcat1,Subcat2)
                             
                            ),  % This is a hack to constrain the list length.
                                                                 % Without this constraint the old TRALE loops.
                                                                 % Somehow the old trale ignores the constraint above,
                                                                 % when it applys unrealized_args and therefore applies to
                                                                 % traces which do not have a subcat value. The two
                                                                 % unrealized_args seem to deblock each other infinitely.
                     marked_:Marked),
                cont:Cont),
           nonloc:Nonloc),
   non_head_dtrs:[(synsem:(Spec,
                           loc:(cat:(head:Head,
                                     xarg:XArg1,
                                     spr:Spr1,
                                     subcat:Subcat1,
                                     marked_:Marked),
                                idiomatic:minus),
                           nonloc:Nonloc)),
                  (synsem:(loc:(cat:(head:(coord,
                                           spec:Spec),
                                     subcat:%list_of_spirits,
%                                     This does not work with coordination, since the coordination rule acesses the first element
% in subcat and hence undelays the constraint and hence the last element in SUBCAT of the coord-phrase gets
% instantiated as a nominal pro.
%
% 22.09.2010

                                            [(arg:loc:cat:(spr:Spr2,
                                                           xarg:XArg2,
                                                           subcat:Subcat2),
                                              realized:plus)]),
                                cont:Cont,
                                idiomatic:minus)))]).




x_conj_y ## (@x_conj_y_coord_phrase,
            non_head_dtrs:[Dtr1,Dtr2])
  ===>
cat> Dtr1,
cat> Dtr2.




% This does not work if it is called for two lists independently. So we integrate the two calls into one
% constraint.
fun unrealized_args(+,-).
unrealized_args(L1,L2) if
   when( (L1=(e_list;ne_list);
          L2=(e_list;ne_list)),
         undelayed_unrealized_args(L1,L2) ).

undelayed_unrealized_args([],[]) if true.
undelayed_unrealized_args([(Arg,
                            realized:minus)|T1],[Arg|T2]) if 
   unrealized_args(T1,T2).


undelayed_unrealized_args([realized:plus|T1],T2) if 
   unrealized_args(T1,T2).


% Returns a list of unrealized arguments that can be found in both lists.
fun unrealized_args(+,+,-).
unrealized_args(L1,L2,L3) if
   when( (L1=(e_list;ne_list),
          L2=(e_list;ne_list)),
         undelayed_unrealized_args(L1,L2,L3) ).

undelayed_unrealized_args([],[],[]) if true.

% The following two clauses have to mention at least on element, since
% otherwise we would get spurious ambiguities, since [] is of course also a
% list of spirits.

% Since they are not availible at higher nodes anyway, they cannot be raised.
% I am not sure whether this makes the right predictions.
% Check! St. Mü. 12.08.2010

% Einen Aufsatz gelesen und einen Report geschrieben hat er nicht.
% Ein Aufsatz gelesen und ein Report geschrieben wurde nicht.

undelayed_unrealized_args([],[(realized:plus,
                               raised:minus)|L2],[]) if list_of_spirits(L2).
undelayed_unrealized_args([(realized:plus,
                               raised:minus)|L1],[],[]) if list_of_spirits(L1).

undelayed_unrealized_args([(Arg,
                            realized:minus)|T1],[(Arg,
                                                  realized:minus)|T2],[Arg|T3]) if 
   unrealized_args(T1,T2,T3).

undelayed_unrealized_args([(realized:plus,
                               raised:minus)|T1],[(realized:plus,
                                                   raised:minus)|T2],T3) if 
   unrealized_args(T1,T2,T3).

% The following two clauses allow to remove realized arguments from the beginnings of
% the first and second list. Unrealized arguments are keept. The have to be matched against
% respective arguments in the other list at a certain point.
undelayed_unrealized_args([(realized:plus,
                               raised:minus)|T1],[(Arg,
                                                   realized:minus)|T2],T3) if 
   unrealized_args(T1,[Arg|T2],T3).

undelayed_unrealized_args([(Arg,
                            realized:minus)|T1],[(realized:plus,
                                                  raised:minus)|T2],T3) if 
   unrealized_args([Arg|T1],T2,T3).



% This computes a new element that is visible externally. It can have a cumulative index.
%
% Sahra reads a book and Max dances, don't they? -> Sahra + Max = plural
%
% Right now only the nonlocal values are computed here. All other unification happens
% in the unification of unrealized arguments that is computed above. The computation above only
% affects unrealized arguments, but here we also include realized XARGS:

% Lieber Stefan
% > Liebe Philippa,
% >
% > Ich denke gerade über Question-Tags nach. Wie sieht es mit den folgenden
% > Beispielen aus:
% >
% >    Sahra kissed him and he likes  her, didn't she?
% >    Sahra kissed him and he likes  her, didn't  he?
% >    Sahra kissed him and he kissed her, didn't she?
% >

% erster Block = (1) und (2) ungrammatisch. Ich denke, dass der 3. Satz geht. Eventuell hängt es ein Bisschen von der Betonung des Tags ab.
% Aber ist es klar, man fragt nur nach dem ersten Konjunkt.
% >    Sahra kissed him and he kissed her, didn't they?
% >    Sahra laughed and Sahra giggled, didn't she/they?
% >
% > Der erste Block ist ungrammatisch, oder? Der zweite müsste OK sein. Beim
% > zweiten Beispiel hängt es davon ab, ob wir über eine oder zwei Sahras
% > sprechen.

% Der erste Satz von diesem 2. Block geht...aber es ist auch nicht ganz normal.
% bei zweitem Satz ist es wie du sagst.



fun accumulated_args(+,+,-).
accumulated_args(L1,L2,L3) if
   when( (L1=(e_list;ne_list),
          L2=(e_list;ne_list)),
         undelayed_accumulated_args(L1,L2,L3) ).

undelayed_accumulated_args([],[],[]) if true.
undelayed_accumulated_args([arg:nonloc:Nonloc],[arg:nonloc:Nonloc],[arg:nonloc:Nonloc]) if true.







mspec_sup_type(S1,S2,SM) :-
        deref(S1,_Ref1,T1),
        deref(S2,_Ref2,T2),
        findall(A,(sub_type(A,T1),sub_type(A,T2)),X),
        most_specific(X,bot,MT),
        deref(S3,_Ref3,MT),
        ud(SM,S3).

most_specific([],T,T).
most_specific([T1|Rest],TI,TOut) :-
        (sub_type(TI,T1) ->
            most_specific(Rest,T1,TOut)
        ; otherwise ->
            most_specific(Rest,TI,TOut)
        ).
