% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%       $RCSfile: core-syntax.pl,v $
%%      $Revision: 73 $
%%          $Date: 2007-03-25 10:52:16 +0200 (Sun, 25 Mar 2007) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: All the syntactic constraints that belong to
%%                 Universal Grammar.
%%       Language: Trale
%%         System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.


% Danish definite marking
cont_identical_lr :=
(synsem:loc:cont:Cont,
 dtrs:[synsem:loc:cont:Cont]).

hcons_identical_lr :=
(h_cons:HCons,
 dtrs:[h_cons:HCons]).


% Persian possessives: The input is a noun,
% the output is a noun with the same semantics, but the RELS list
% contains the possessive relation.
cont_hcons_identical_lr :=
 (@cont_identical_lr,
  @hcons_identical_lr).

rels_identical_lr :=
(rels:Rels,
 subtract_rels:SR,
 dtrs:[(rels:Rels,
        subtract_rels:SR)]).

% needed in Persian and German for handling lexical rules that introduce adjuncts.
% The CONT comes from the selected adjunct, RELS and HCONS are passed to the output.
% St. Mü. 23.07.2010
rels_hcons_identical_lr :=
 (@rels_identical_lr,
  @hcons_identical_lr).


sem_identical_lr :=
(@cont_hcons_identical_lr,
 @rels_identical_lr).

sem_identical_unsared_index :=
 (@rels_identical_lr,
  synsem:loc:cont:(gtop:GTop,
                   ltop:LTop),
  h_cons:HCons,
  dtrs:[(synsem:loc:cont:(gtop:GTop,
                          ltop:LTop),
         h_cons:HCons)]).


% The index may differ in the case of nominalizations.
% Furthermore, depictives differ in their indices.
scoping_lr_unshared_index :=
( %complex_word
  @ltop_lbl_le,
  @arg0_le,
  synsem:loc:cont:(gtop:GTop,
                   lextop:Lextop),
  rels:[arg3:LTop|Rels],
  subtract_rels:SR,
  h_cons:H_Cons,
  dtrs:[( synsem:loc:cont:(ltop:LTop,
                           gtop:GTop,
                           lextop:Lextop),
          rels:Rels,
          subtract_rels:SR,
          h_cons:H_Cons)]).

scoping_lr :=
 (@scoping_lr_unshared_index,
  synsem:loc:cont:ind:Ind,
  dtrs:hd:synsem:loc:cont:ind:Ind).


scoping_lextop_lr :=
 (@lextop_lbl_le,
  @arg0_le,
  @relational_le,   % Key points to first element in RELS
  synsem:loc:cont:(ltop:LTop,
                   gtop:GTop,
                   ind:Ind),
  rels:[arg1:LexTop|Rels],
  subtract_rels:SR,
  h_cons:H_Cons,
  dtrs:[( synsem:loc:cont:(ltop:LTop,
                           lextop:LexTop,
                           gtop:GTop,
                           ind:Ind),
          rels:Rels,
          subtract_rels:SR,
          h_cons:H_Cons)]).

