% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: core-syntax-spirits.pl,v $
%%  $Revision: 314 $
%%      $Date: 2008-05-03 22:28:07 +0200 (Sa, 03 Mai 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: A grammar for Maltese
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.


% Diese Beschränkung schließt die Kombination von teilweise gesättigten
% VPen mit optional kohärent konstruierenden Verben aus.
% Da Determinatoren über SPR selegiert werden, sind Fälle wie
% "vom Buch", in denen eine Präposition mit einer N' kombiniert wird,
% nicht betroffen.

% Hey, and what happens with coordination? St. Mü. 16.02.2009

% head_argument_phrase *> non_head_dtrs:hd:synsem:loc:cat:subcat:list_of_spirits.

% This also works for coordination.
% The conjunction is a minor head that takes arguments.
% These may be incomplete. All other heads (complementizers for instance) take care
% of the saturation of their arguments themselves.
%
% St. Mü. 11.06.2010
%
(head_argument_phrase,
 synsem:loc:cat:head:major) *> non_head_dtrs:hd:synsem:loc:cat:subcat:list_of_spirits.

%% Kopf-Adjunkt-Strukturen


head_adjunct_phrase *>
   (head_dtr:synsem:HeadSynsem,
    non_head_dtrs:[synsem:(loc:cat:(head:mod:HeadSynsem,
                                    subcat:list_of_spirits),
                           lex:minus)]).


%% Spezifikatorprinzip

(headed_phrase,
 non_head_dtrs:[synsem:loc:cat:head:spec:synsem]) *>
   (head_dtr:synsem:Head,
    non_head_dtrs:[synsem:loc:cat:head:spec:Head]).


head_non_specifier_struc *> 
   (synsem:loc:cat:spr:Spr,
    head_dtr:synsem:loc:cat:spr:Spr).


% This is not true for configurational languages like Danish and English,
% in which the specifier of a participle is filled, but will be raised by the
% governing auxiliary. St. Mü. 12.10.2009
%spr:ne_list *> spr:hd: @non_raised.


% are there any languages that allow for multiple specifiers, appart from Danish?
% If not we may get away with requiring a fixed order here.
head_specifier_phrase *> 
   (synsem:loc:cat:spr:append(FirstPart,
                               [(arg:Spr,
                                 raised:Raised,
                                 realized:plus)|Rest]),  % we keep the element around, since some languages look at their subject/spr.
                                                         % for instance Danish inserts an expletive, if the subject is extracted. 12.07.2011, St. Mü.
       
                                                 % only Danish allows multiple elements in SPR. Specify implicational constraints
                                                 % for other languages! St.Mü. 28.06.2011
    head_dtr:synsem:loc:cat:(spr:append((FirstPart,
                                         list_of_spirits),
                                        [(@non_raised,     % This is needed here since languages like Chinese and Maltese allow for direct
                                                           % predication without copula. Alternatively one could integrate this in the root nodes.
                                                           % This would make it necessary to project the SPR as a spirit.
                                   
                                          arg:Spr,
                                          raised:Raised,
                                          realized:minus)|Rest]),
                             subcat:list_of_spirits),
    non_head_dtrs:[synsem:Spr]).




% Adjunkte sind Extraktionsinseln
(headed_phrase,
 non_head_dtrs:hd:synsem:trace:extraction) *>
      head_dtr:synsem:loc:cat:head:mod:none.



% The specifier of the head daughter is underspecified.
% This allows binding off of the nonlocal dependency before
% the subject or after the subject in Maltese.
head_filler_phrase *>
   (head_dtr:synsem:(loc:cat:head:verb,
                             % subcat:list_of_spirits\),     % Not true for Persian. St. Mü. 23.09.2010
                     
                     nonloc:to_bind:slash:[Slash]),
    non_head_dtrs:[(%phon:ne_list,
                    % to do: there has to be a restriction on the filler
                    %        that it is definite.
                    synsem:(loc:Slash,
                            nonloc:inher:(rel:[], % "In der hat er geschlafen." hätte sonst zwei Strukturen.
                                          slash:[])))]).



fun list_of_spirits(-).
list_of_spirits(List) if
       when( List=(e_list;ne_list)
           , undelayed_list_of_spirits(List)
           ).

undelayed_list_of_spirits([]) if true.


realized_as_dropped_pronominal :=
 (realized:plus,
  arg:(loc:(cat:subcat:[],       % ansonsten hängt ein Constraint rum
            cont:pron),
       nonloc:inher:(rel:[],
                     slash:[]),
       lex:minus)).

/*
% pro drop and optional arguments
undelayed_list_of_spirits([@realized_as_dropped_pronominal
                          |Rest]) if !, % hey, defaults!
                                  list_of_spirits(Rest).

*/

undelayed_list_of_spirits([realized:plus|Rest]) if list_of_spirits(Rest).




fun list_of_unrealized_args(-).
list_of_unrealized_args(List) if
       when( List=(e_list;ne_list)
           , undelayed_list_of_unrealized_args(List)
           ).

undelayed_list_of_unrealized_args([]) if true.


undelayed_list_of_unrealized_args([realized:minus|Rest]) if list_of_unrealized_args(Rest).
