% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          $File: german-maltese-lexrules.pl $
%%      $Revision: 94 $
%%          $Date: 2007-03-26 17:16:20 +0200 (Mon, 26 Mar 2007) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: Macros that are identical for German and Maltese
%%       Language: Trale
%          System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile (lex_rule)/2.
:- discontiguous (lex_rule)/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.



non_aux_verb :=
 synsem:loc:cat:head:aux:minus.


% all German verbs except modals, `helfen', `haben', and decomposed verbs
non_flip_verb_le :=
 (@event_verb_le,
  @singleton_rel_le,
  @lexical_event_verb_type, % depends on the language, whether this is event_verb_root, event_verb_stem, or event_verb_word
  @non_flip_verb).          % Persian and Dutch differ from German in what triggers Flip. This is a parameter.
                            % Having it and having to mention non_flip_verb in all the subtypes is ugly since the constraint
                            % is only relevant for German, but I do not know a better way right now. St. Mü. 10.10.2007




 
intrans_verb :=
 (@non_flip_verb_le,
  @non_aux_verb,
  @incoherent_le,        % although this is not done, control verbs should inherit from intrans_verb
                         % if this is done, the constraint has to be put somewhere else, that is further down the hierarchy.
                         % the constraint is needed for speed only. St. Mü. 14.02.2010
     
  @non_idiomatic_le).    % for testing, to be changed later


intrans_unacc_verb :=
  (@intrans_verb,
   @unacc_verb,
   synsem:loc:cat:arg_st:append(_,[@np_str(Ind)]),  % Do not know, why `last' does not work here. Trale bug?
   rels:[arg2:Ind]).

%          goal last(Arg_st,@np_str(Ind)).



intrans_nerg_verb :=
 (@intrans_verb,
  @nerg_pred,
  synsem:loc:cat:arg_st:append(_,[@np_str(Ind)]),  % Do not know, why last does not work herer. Trale bug? last(@np_str(Ind)),
  rels:[arg1:Ind]).

%       goal last(Subcat,@np_str(Ind)).

% schlafen
strict_intrans_verb :=
  (@intrans_verb,
   @non_hcons_le,
   synsem:loc:cat:arg_st:[_]).

strict_intrans_nerg_verb  :=
  (@strict_intrans_verb,
   @intrans_nerg_verb).

strict_intrans_unacc_verb :=
  (@strict_intrans_verb,
   @intrans_unacc_verb).

intrans_verb(Relation) :=
 (@strict_intrans_nerg_verb,
  rels:[pred:Relation]).

intrans_verb(Relation,Sort) :=
 (@strict_intrans_nerg_verb,
  rels:[(pred:Relation,
         arg1:sort:Sort)]).

intrans_verb(Relation-(a_ _),Arg1Sort-anything,_AuxF-auxf,_VerbSort-situation,_Aktionsart) :=
  @intrans_verb(Relation,Arg1Sort).

intrans_unacc_verb(Relation) :=
 (@strict_intrans_unacc_verb,
  rels:[pred:Relation]).

intrans_unacc_verb(Relation,Sort) :=
 (@strict_intrans_unacc_verb,
  rels:[(pred:Relation,
         arg2:sort:Sort)]).


% denken an
np_pp_verb := 
 (@intrans_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ @pp(Ind2), _ ],
  rels:[arg3:Ind2]).

np_pp_nerg_verb :=
 (@np_pp_verb,
  @intrans_nerg_verb).

% Persian
np_pp_verb(PForm,Relation,Sort1,Sort3) :=
 (@np_pp_nerg_verb,
  synsem:loc:cat:arg_st:hd: @pp_pform(PForm),
  rels:[(pred:Relation,
         arg1:sort:Sort1,
         arg3:sort:Sort3)]).

% German
np_pp_verb(PForm,Case,Relation) :=
 (@np_pp_nerg_verb,
  synsem:loc:cat:arg_st:hd: @pp(PForm,Case),
  rels:[pred:Relation]).


% helfen
np_np_nerg_verb :=
 (@intrans_nerg_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ @np(Ind2), _ ],
  rels:[arg2:Ind2]).

% helfen
np_np_verb(Case,Relation) :=
 (@np_np_nerg_verb,
  synsem:loc:cat:arg_st:hd: @np(Case,_Ind2),
  rels:[pred:Relation] ).

% glauben
np_s_nerg_verb :=
 (@intrans_nerg_verb,
  @scopal_event_embedding_verb,
  synsem:loc:cat:arg_st:[ @s, _ ]).

% Danish: CP oder SVO
np_cp_s_nerg_verb :=
 (@intrans_nerg_verb,
  @scopal_event_embedding_verb,
  synsem:loc:cat:arg_st:[ @cp_s_uninv, _ ]).

np_interrog_s_nerg_verb :=
 (@intrans_nerg_verb,
  @scopal_event_embedding_verb,
  synsem:loc:cat:arg_st:[ @interrog_s, _ ]).

np_s_verb(Relation) :=
 (@np_s_nerg_verb,
  rels:[pred:Relation] ).

np_s_verb(Relation,_Sort1,_Sort2) :=
 (@np_s_nerg_verb,
  rels:[pred:Relation] ).

np_cp_s_verb(Relation) :=
 (@np_cp_s_nerg_verb,
  rels:[pred:Relation] ).

np_cp_s_verb(Relation,Sort1) :=
 (@np_cp_s_nerg_verb,
  rels:[(pred:Relation,
         arg1:sort:Sort1)] ).

np_interrog_s_verb(Relation,Sort1) :=
 (@np_interrog_s_nerg_verb,
  rels:[(pred:Relation,
         arg1:sort:Sort1)] ).

% gelingen
np_np_unacc_verb := 
 (@intrans_unacc_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ @np(Ind2), @np_str(Ind1) ],
  rels:[(arg2:Ind1,
         arg3:Ind2)]).

np_np_unacc_verb(Case,Relation) :=
 (@np_np_unacc_verb,
  synsem:loc:cat:arg_st:hd: @np(Case,_Ind2),
  rels:[pred:Relation]).



% lieben, geben
trans_verb :=
 (@nerg_pred,
  @non_flip_verb_le,
  @non_aux_verb,
  @incoherent_le,        % although this is not done, control verbs should inherit from intrans_verb
                         % if this is done, the constraint has to be put somewhere else, that is further down the hierarchy.
                         % the constraint is needed for speed only. St. Mü. 14.02.2010
     
  synsem:loc:cat:arg_st:append(_, [ @np_str(Ind2), @np_str(Ind1) ]),
  rels:hd:(arg1:Ind1,
           arg2:Ind2)).


% This is the same as trans verb, but with to realtions for decomposable verbs like `open':
%
% He opened the window again.
%
% Cause is the second relation. The first relation (for instance `open') is linked to the
% local TOP. Therefore adverbs and quantifiers scope over this element and are free to scope over
% the higher CAUSE predicate. But the required constraints refer to the LABEL of the first
% predicate.
% St. Mü. 09.06.2008
decomposed_trans_verb :=
 (@decomposed_le,
  @nerg_pred,
  %@non_flip_verb_le,  We cannot inherit this, since arg0 is the event contributed by the second relation.
     @lexical_event_verb_type, 
     @non_flip_verb,
  @non_aux_verb,
  synsem:loc:cat:arg_st:append(_, [ @np_str(Ind2), @np_str(Ind1) ]),
  rels:[(% open_rel,
           lbl:Label2,
           arg2:Ind2),
        (% cause_rel,
           arg1:Ind1,
           arg3:Label1)
        ],
  h_cons:[(qeq,
           harg:Label1,
           larg:Label2)]).


% lieben
strict_trans_verb :=
 (@trans_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ _, _ ]).


non_idiomatic_trans_verb :=
 (@non_idiomatic_le,
  @trans_verb,
  rels:tl:e_list).

non_idiomatic_strict_trans_verb :=
 (@non_idiomatic_trans_verb,
  @strict_trans_verb).

trans_verb(Relation) :=
 (@non_idiomatic_strict_trans_verb,
  rels:[pred:Relation]).

trans_verb(Relation,Sort1,Sort2) :=
 (@non_idiomatic_strict_trans_verb,
  rels:[(pred:Relation,
         arg1:sort:Sort1,
         arg2:sort:Sort2)]).

trans_verb(Relation-(a_ _),Arg1Sort,Arg2Sort,_AuxF,_VerbSort,_Aktionsart) :=
 @trans_verb(Relation,Arg1Sort,Arg2Sort).


idiomatic_trans_verb(IdiomaticRelation,IdiomaticObjectRelation,NonIdiomaticObjectRelation,Sort1,Sort2) :=
 (@strict_trans_verb,
  rels:[(pred:IdiomaticRelation,
         arg1:sort:Sort1,
         arg2:(Ind2,
               sort:Sort2)),
        (pred:IdiomaticObjectRelation,
         lbl:ObjectLbl,
         arg0:Ind2)],
  subtract_rels:[(pred:NonIdiomaticObjectRelation,
                  lbl:ObjectLbl,
                  arg0:Ind2)]).


% These are always non_idiomatic.
strict_decomposed_trans_verb :=
 (@decomposed_trans_verb,
  @non_idiomatic_le,
  synsem:loc:cat:arg_st:[ _, _ ]).

caused_trans_verb(Relation,Sort1,Sort2) :=
 (@strict_decomposed_trans_verb,
  rels:[(pred:Relation,
         arg2:sort:Sort2),
        (cause_rel,
         arg1:sort:Sort1)]).

% geben in German and giv in Danish
ditrans_verb_le *>
 (@non_idiomatic_trans_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ @np(Ind3), _, _ ],
  rels:[arg3:Ind3]).

ditrans_verb(Relation) :=
 (ditrans_verb_le,
  rels:[pred:Relation]).

ditrans_verb(Relation,Sort1,Sort2,Sort3) :=
 (ditrans_verb_le,
  rels:[(pred:Relation,
         arg1:sort:Sort1,
         arg2:sort:Sort2,
         arg3:sort:Sort3)]).

/*
opaqe_verb_le :=
  (@non_idiomatic_strict_trans_verb,
   

% seek, suchen, ...
opaqe_verb(Relation,Sort1,Sort2,Sort3) :=
 (@opaqe_verb_le,
  rels:[(pred:Relation,
         arg1:sort:Sort1,
         arg2:sort:Sort2,
         arg3:sort:Sort3)]).

*/


% Persian give
np_np_pp_verb :=
 (@non_idiomatic_trans_verb,
  @non_hcons_le,  
  synsem:loc:cat:arg_st:[ @pp(Ind3), _, _ ],
  rels:[arg3:Ind3]).

np_np_pp_verb(Relation,PForm-pform,Sort1,Sort2,Sort3) :=
 (@np_np_pp_verb,
  synsem:loc:cat:arg_st:hd: @pp_pform(PForm),
  rels:[(pred:Relation,
         arg1:sort:Sort1,
         arg2:sort:Sort2,
         arg3:sort:Sort3)]).



% Komplementierer und das Verb in Erststellung
complementizer_like_word *>
 (@spr_saturated,
  @non_idiomatic_le,
   synsem:loc:cat:(head:(        %dsl:none,  German and Danish only
                         initial:plus),
                   subcat:[ (@argument_xp,
                             arg:( loc:cat:(head:(verb,
                                                  vform:fin,
                                                  %initial:minus, German and Persian, but not Danish
                                                  inv:minus),
                                           subcat:list_of_spirits)
                                   %trace:minus_or_head_movement  Persian does not have head movement. Hindi?
                                 )) ])).


% The sharing of the LTop has to be done here since the verb movement rule
% does not share the LTop with the argument, but with the rule input instead.
% This is necessary to get scopal modifiers right: The LTop is percolated down to the
% verb trace via DSL. If the VP contains scopal modifiers, the LTop changes to the one
% of the modifier. If we would take the LTop of the selected argument as ARG3,
% we would get circular structures, which would not scope.
% St. Mü. 11.03.2008

% In the Danish preposed negation construction the verum_rel can scope between the
% contribution of the complementizer and the contribution of the embedded clause.
% Hence we need a qeq instead of identification of arg3 with the complement's LTOP.
% St. Mü. 26.06.2009
complementizer_word *>
 (%simple_word,
  %complementizer_like_word,
  @overt_le,
  @non_rel_le,
  @non_slashed_le,
  @non_relativizer_le,
  @ltop_lbl_le,
  @arg0_le,
  synsem:loc:cat:(head:comp,
                  %arg_st:hd:arg:loc:cat:head:dsl:none     German and Danish only
                  subcat:hd:arg:loc:cont:ltop:SLTop),
  rels:[arg1:SHandle],
  h_cons:hd:(qeq,
             harg:SHandle,
             larg:SLTop)
  ).


comp_complementizer_word *>
 (@simple_le,                           % This links the GTOP of the clausal complement to the GTOP of the complementizer.
                                        % Adjunct complementizers differ in introducing a new GTOP -> quantifiers cannot scope out
                                        % of the adjunct clause.
  synsem:loc:cat:(head:comp_comp,
                  subcat:hd:arg:loc:cat:spr:list_of_non_move_arguments),
  rels:[assertion],
  h_cons:tl:e_list).

mod_complementizer_word *>
  @scopal_complex_v_modifier_word.
%   synsem:loc:cat:arg_st:hd:arg:nonloc:inher:slash:[]).  % adjuncts are islands (for various reasons ...)
                                                         % informations structure and processing
                                                         % reduces the number of edges from 230 to 194 for
                                                         % Peter arbejder, hvis han ikke læser en bog. (Danish)

% This does not work, since the preposed negation construction in Danish identifies the complement of the modifer
% with an S from which a negated element is extracted.


complementizer(CForm) :=
 (comp_complementizer_word,
  synsem:loc:cat:head:cform:CForm).

mod_complementizer(Relation) :=
 (mod_complementizer_word,
  rels:[pred:Relation]).
