% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          $File: german-maltese-lexrules.pl $
%%      $Revision: 94 $
%%          $Date: 2007-03-26 17:16:20 +0200 (Mon, 26 Mar 2007) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: Macros that are identical for German and Maltese
%%       Language: Trale
%          System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile (lex_rule)/2.
:- discontiguous (lex_rule)/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inflection LR for non-finite verbs (non-finite past form in Persian
% bse and zu-inf in German.
%
% The subject is taken out of the SUBCAT-List
% and represented under SUBJ.

% German, Persian
subject_in_subj_verb_infl_lr *>
 (%complex_word
  @sem_identical_lr,
  synsem:(loc:cat:(head:(verb,
                         initial:Initial, % all verbs are initial:minus in Persian
                         vform:VForm,
                         subj:Subj,
                         da:DA,
                         mod:Mod),
                   spr:[],
                   subcat:block_subject(Subcat,Subj)),
          nonloc:Nonloc,
          lex:Lex,
          trace:Trace),
  dtrs:[(stem,                     % Persian: the past stem, present is a root.
         synsem:(loc:cat:(head:(verb,
                                initial:Initial,
                                vform:VForm,
                                da:DA,
                                mod:Mod),
%                          spr:Spr,
                          arg_st:Subcat),
                 nonloc:Nonloc,
                 lex:Lex,
                 trace:Trace)) ]).


inf_verb_infl_lr *>
 (%bse_or_inf_verb_infl_lr, for German
  %da_blocking_verb_infl_lr for Persian
  synsem:loc:cat:head:vform:inf).


% Participle forms in Danish, German and Persian,
% -an infinitives that are used in -i derivations.

% Danish adds an expletive in the mapping from ARG-ST to SUBCAT and SPR,
% while German and Persian take everything over from ARG-ST to SUBCAT

da_blocking_verb_infl_lr *>
( %complex_word,
  @sem_identical_lr,
  synsem:(loc:cat:(head:(verb,
                         inv:(Inv,
                              minus),
                         initial:Initial,
                         vform:VForm,
                         subj:DA,
                         da:DA,
                         mod:Mod),
                   arg_st:block_da(DA,ArgSt)),
          nonloc:Nonloc,
          lex:Lex,
          trace:Trace),
  dtrs:[(stem,
         synsem:(loc:cat:(head:(verb,
                                inv:Inv,
                                initial:Initial,
                                vform:VForm,
                                da:DA,
                                mod:Mod),
                          arg_st:ArgSt),
                 nonloc:Nonloc,
                 lex:Lex,
                 trace:Trace))]).


ppp_verb_infl_lr *>
 (%da_blocking_verb_infl_lr
  synsem:loc:cat:head:vform:ppp).