% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          $File: german-maltese-lexrules.pl $
%%      $Revision: 94 $
%%          $Date: 2007-03-26 17:16:20 +0200 (Mon, 26 Mar 2007) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: Macros that are identical for German and Maltese
%%       Language: Trale
%          System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile (lex_rule)/2.
:- discontiguous (lex_rule)/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Pränominale Partizipien (`geliebte' und `zu lesende')
%
%

% This rule does not apply to unergative verbs, since their DA got removed
% from the SUBCAT list and nothing is left that matches the np_str description
% for Subj below. This explains why "der geschlafene Mann" is out and why
% impersonal passives in Persian are out. zadan governs an NP and a PP.
% The NP is the designated argument, so nothing can be blocked in participle
% formation.

% The zu-inf LR in German adds a modal component.

% This is used to derive participles in German and -i adjectives in Persian.

adj_derivation_lr *>
 (%derived_stem, in German
  %complex_word in Persian
  synsem:(loc:cat:(head:(adj,
                         scopal:minus,
                         subj:raise([Subj]),
                         da:(DA,
                             list_of_blocked_arguments)),
                   spr:Spr,
                   subcat:(                                         % unfortunately the value of Subcat maybe underspecified
                                                                    % therefore we have to say that the elements have to be arguments.
                                                                    % this excludes empty determiners as arguments of participles.
                           %list_of_spec_none,  % did not make a difference, maybe with the preverb under SUBJ.
                           raise(Subcat))),
          nonloc:Nonloc,
          lex:Lex,
          trace:Trace),
  dtrs:[(word,
         synsem:(loc:cat:(head:(verb,
                                da:DA,
                                flip:minus % schließt Ersatzinfinitive aus * `wollene' statt `gewollte'
                               ),    
                          spr:Spr,
                          subcat:append(Subcat,[(Subj,@np_str)])),

% This case would be needed to account for passives with a nominal preverb.
% I did not succeed parsing with this, since the valence list of the passive auxiliary + LV
% is not instantiated enough. Don't know how to fix this. St. Mü. 12.11.2007
%                         (
%                                 [(Subj,@arg(loc:cat:head:case:case_type:str))|(%list_of_non_nominatives,
%                                                   Subcat)])),
                 nonloc:Nonloc,
                 lex:Lex,
                 trace:Trace),
         % Dies ist ein Hack, der unechte Mehrdeutigkeiten für `angelachte' und `anzulachende' ausschließt.
         % Ohne diese Beschränkung kann die Partikel mit dem Verb kombiniert werden, bevor die Derivation
         % angewendet wird. Dies läßt sich nicht ohne weiteres ausschließen, da man die entsprechende
         % Partizipform für `Sie hat ihn angelacht.' braucht. Allerdings braucht man nach der Derivation
         % auch eine syntaktische Komplexbildung mit Verben und Resultativpräsikaten:
         %
         % die sturmreif geschossene Stadt (aber auch: das plattgefahrene Tier)
         % der schlafen wollende Mann
         %
         % Die Beschränkung funktioniert nur, weil die Affixe nicht in den Tochterlisten verwaltet werden.
         dtrs:tl:e_list)]).

/*
% The problematic case from Persian.
% The preverb provides the arguments and is represented under SUBJ. Therefore
% the SUBCAT list is absolutely underspecified. 
(ppp_part_derivation_lr,
 synsem:loc:cat:head:subj:hd:arg:loc:cluster:plus) *>
 synsem:loc:cat:(head:subj:hd:arg:loc:cat:head:subj:[], % plays a role when argument lists are appended
                 subcat:(e_list
                        ;tl:e_list
                        ;tl:tl:e_list)).

*/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Partizip II
%
% geliebt (V) -> geliebt (A)
%
% Flexionsregel erzeugt dann: geliebt (A) -> geliebte
%
% The state passive takes the verbal participle form, hence we have to block
% a reading in which the adjectival particple is used predicatively with the copula.
% This is ensured by requiring the adjectivel participle to modify something (MOD).
%
ppp_part_derivation_lr *>
 (%adj_derivation_lr,
  @sem_identical_lr,
  %synsem:loc:cat:head:mod_adj,  % German only
  dtrs:[ synsem:loc:cat:head:vform:ppp ]).


ppp_part_derivation_lr ##
  Dtr
 **>
( ppp_part_derivation_lr,
  dtrs:[Dtr]
  )
morphs
  X becomes X.
