% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: core-syntax-spirits.pl,v $
%%  $Revision: 302 $
%%      $Date: 2008-04-21 17:24:18 +0200 (Mo, 21 Apr 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Case assignment for all nominative-accusative
%%             languages.
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.



% What about finite verbs in Maltese and Chinese? 26.03.2007


% Das ist die erste Version gewesen, leider ist sie aber mit der scheinbar mehrfachen Vorfeldbesetzung
% nicht kompatibel. Wenn die MVF als Instanz der kohärenten Konstruktion analysiert werden soll, dann
% ist es für die Analyse von Sätzen wie (1) nötig, die Anhebung von Argumenten eines finiten Verbs zuzulassen.
%
% (1) Der Frau das Buch gibt er nicht.
%
%(word,
% synsem:loc:cat:head:vform:fin) *> synsem:loc:cat:subcat:list_of_non_raised_arguments.

% Bei der folgenden Formulierung wird der raised-Wert erst festgelegt, wenn das Verb in einer
% syntaktischen Struktur verwendet wird.

% head_dtr:(%(complex_word;
%           %  simple_word),
%           complex_word,
%            % word, % should be sufficient, seems to be a TRALE bug.
%           phon:ne_list, % sonst würde die finite Spur im Vorfeld die Anhebung eines Arguments nicht gestatten.
%                         % Leider wird die Implikation somit auch nicht auf die Verbspur in der rechten Satzklammer
%                         % im Hauptsatz angewendet. Deshalb muß die Beschränkung noch mal extra in der
%                         % Lexikonregel für die Bewegung des finiten Verbs formuliert werden.

%           synsem:loc:cat:head:vform:fin) *> synsem:loc:cat:subcat:list_of_non_raised_arguments.


% Even this is to general, since in Persian we have the progressive, in which a finite verb is embedded.
% So we add the cluster:minus constraint. [which is inappropriate for Danish ...] in Danish the constraint is
% included in the lexical rule that forms finite verbs.

/*

head_dtr:(%(complex_word;
          %  simple_word),
          complex_word,
           % word, % should be sufficient, seems to be a TRALE bug.
          phon:ne_list, % sonst würde die finite Spur im Vorfeld die Anhebung eines Arguments nicht gestatten.
                        % Leider wird die Implikation somit auch nicht auf die Verbspur in der rechten Satzklammer
                        % im Hauptsatz angewendet. Deshalb muß die Beschränkung noch mal extra in der
                        % Lexikonregel für die Bewegung des finiten Verbs formuliert werden.

          synsem:loc:cat:head:vform:fin)) *> synsem:loc:cat:subcat:list_of_non_raised_arguments.

*/

fun assign_case_verb(-).

%assign_case_verb(_) if !,prolog((write(yes-yes),nl)),true. % for debugging we can disable case assignment here.

% Da Spuren auch Kasus zuweisen, muß man warten, bis die SUBCAT-Liste überhaupt instantiiert
% ist. Ansonsten, wäre klar, daß es immer ein Argument gibt und man könnte gleich mit
% assign_case_verb2 beginnen.
assign_case_verb(List) if
       when( List=(e_list;ne_list)
           , assign_case_verb2(List)
           ).



assign_case_verb2([(H,
                    raised:R)|T]) if
       when( (R=(plus;minus),
              T=(e_list;ne_list))
           , undelayed_assign_case_verb([H|T])
           ).

% Wenn die Argumente nicht lokal realisiert wurden, passiert nichts.
undelayed_assign_case_verb([raised:plus])      if true.
undelayed_assign_case_verb([raised:plus|Rest]) if assign_case_verb(Rest).

% Wenn die Argumente lokal realisiert wurden, weise NPen mit strukturellem
% Kasus ja nach Position Nominativ oder Akkusativ zu.

% Die letzte NP mit strukturellem Kasus bekommt Nominativ.
undelayed_assign_case_verb([(raised:minus,
                             @n_str,
                             arg:loc:cat:head:case:morph_case:nom)])  if true.

% Persian verbal nouns have a determiner in SPR. Since Maltese (still) has the subject under SPR,
% the SPR is appended to the list of NPs that get assigned case. Therefore the subject is on second position.
% This clearly is not nice ... St. Mü. 12.01.2008
undelayed_assign_case_verb([(raised:minus,
                             @n_str,
                             arg:loc:cat:head:case:morph_case:nom),@no_noun])  if true.

% Ein Aufsatz wurde ihm zu lesen erlaubt.
undelayed_assign_case_verb([(raised:minus,
                             @n_str,
                             arg:loc:cat:head:case:morph_case:nom),@n_lex])  if true.


% Nouns that are parts of verbal clusters get lexical case.
% They are unsaturated. The following clause accounts both for regular NPs
% with lexical case and for nouns that are part of predicate complexes.
undelayed_assign_case_verb([(raised:minus,
                             @n_lex)])                                  if true.
undelayed_assign_case_verb([(raised:minus,
                             @no_noun)])                                if true.

% Alle anderen NPen mit strukturellem Kasus bekommen Akkusativ.
% Es muß noch mindestens ein Element in der Valenzliste geben, daß
% nicht durch Passivierung blockiert ist (realized:bool = nicht realized:blocked).
% Für dieses werden die obigen Klauseln angewendet.
undelayed_assign_case_verb([(raised:minus,
                             @n_str,
                             arg:loc:cat:head:case:morph_case:acc),(Nom,
                                                                    @n_str)|Rest])  if assign_case_verb([Nom|Rest]).
% Er verspricht ihm das lied zu singen.
undelayed_assign_case_verb([(raised:minus,
                             @n_str,
                             arg:loc:cat:head:case:morph_case:acc),(Lex,
                                                                    @n_lex),
                                                                   (Nom,
                                                                    @n_str)|Rest])  if assign_case_verb([Lex,Nom|Rest]).
undelayed_assign_case_verb([(raised:minus,
                             @n_lex)|Rest])                                          if assign_case_verb(Rest).
undelayed_assign_case_verb([(raised:minus,
                             @no_noun)|Rest])                                        if assign_case_verb(Rest).




