% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: lexrules.pl,v $
%%  $Revision: 223 $
%%      $Date: 2007-09-12 19:27:44 +0200 (Wed, 12 Sep 2007) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: A German grammar using MRS semantics.
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile lex_rule/2.
:- discontiguous lex_rule/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.

/*

The problem with this lexical rule is that it feeds the verb movement lexical rule. If this is not blocked we get:

Bjarne arbejder der.

With the expletive `der' as the subject and the verb moved to initial position and Bjarne fronted.

So instead of this rule, I suggest a phrasal schema. The schema licences a fixed combination and
since we do not have transformations, we do not get the wrong order above.

St. Mü. 25.03.2011

*/

% Danish puts the referential NP into SLASH, German and Yiddish put the expletive into SLASH.

positional_expl_lr *>
( @sem_identical_lr,
  stem,
  synsem:(loc:cat:(head:(Head,
                         vform:fin),                 % restricts the kind of lexical rules that can apply to this stem
                   arg_st:append(ArgSt,[@positional_np_expl])),
          nonloc:Nonloc),
  infl:Infl,
  dtrs:[ (stem,
          synsem:(loc:cat:(head:Head,
                           arg_st:(ArgSt,
                                   last_el(arg:loc:cont:ind:ref))),
                  nonloc:Nonloc),
           infl:Infl)  ]).

positional_expl_lr ##
   Dtr
  **>
( positional_expl_lr,
  dtrs:[Dtr])
morphs
  X becomes X.

/*
% Danish: former subject is extracted
positional_expl_lr *>
 synsem:loc:cat:arg_st:append(_,[arg:(loc:Loc,
                                      nonloc:inher:slash:[Loc],_]).


% German, Yiddish: expletive is extracted
positional_expl_lr *>
 synsem:loc:cat:arg_st:last_el(arg:nonloc:inher:slash:ne_list).

*/