% -*-  coding:utf-8 mode:trale-prolog   -*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          $File: predication-projection.pl $
%%      $Revision: 280 $
%%          $Date: 2008-01-11 15:00:27 +0100 (Fr, 11 Jan 2008) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: a unary branching rule that licences predicational NPs (instead of a LR)
%%       Language: Trale
%          System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- multifile (if)/2.
:- discontiguous (if)/2.
:- multifile (fun)/1.
:- discontiguous (fun)/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile (lex_rule)/2.
:- discontiguous (lex_rule)/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.



% The case value in German can be either agreeing to the
% subject or nominative.
% Lass mich Dein Tanzpartner sein. (Funny van Dannen)
% See Tiersch and Müller, 1999
prd_noun *> case:case_type:lex.

% Hey, we finally got a PredP. =;-) St. Mü. 16.02.2009
pred_np *>
 (synsem:(loc:cat:(head:(prd_noun,
                         %prd:plus,
                         %mod:none,
                         % for German the case is lexical nominative, for some dialects we have case agreeing with the subject.
                         subj:[@np_str(Ind)]),                  
                   spr:Spr,
                   subcat:Subcat),
          nonloc:Nonloc),                            % should be inherited, but currently this is the only unary branching rule
  c_cont:(hook:(event_cont,
                ltop:LTop,
                gtop:GTop,
                lextop:LTop,
                ind:Event),
          rels:[(pred: (a_ equal_rel),
                 lbl:LTop,
                 arg0:(Event,
                       event), % this blocks recursive application of this rule
                 arg1:Ind,
                 arg2:Ind2)]),
  
  non_head_dtrs:[(synsem:(loc:(cat:(head:(noun,
                                          mod:none), % may be PRD + in case of English raising nouns.
                                    spr:(Spr,
                                         list_of_spirits),
                                    subcat:(Subcat,
                                            list_of_spirits)),
                               cont:(npro, % rules out pronouns
                                     gtop:GTop,
                                     ind:(Ind2,
                                          index),
                                     key: @not_(proper_q) % rules out proper names
                                    ),
                               idiomatic:minus), % The semantics principle ignores the semantics of all daughters that
% are idiomatic plus. Currently the feature is used for arguments in the
% predicate cluster only. 
                          nonloc:(Nonloc,
                                  inher:rel:[]), % der Lehrer, der Peter war/dessen Schwester Klara war = identity 
                          trace:minus))]).

% The subject always outscopes the predicate.
% Every candidate is a woman.
% Vx candidate(x) -> (Ey woman(y) ^ x = y)
% wrong: Ey woman(y) ^ (Vx candidate(x) -> x = y) since this implies that there must be a woman, which is not true.

(pred_np,
 synsem:loc:cat:head:subj:hd:arg:loc:cont:npro) *>
(synsem:loc:cat:head:subj:hd:arg:loc:cont:quant:body:Subj,
 c_cont:h_cons:[(qeq,
                 harg:Subj,
                 larg:Pred)],
 non_head_dtrs:hd:synsem:loc:cont:quant:lbl:Pred).

(pred_np,
 synsem:loc:cat:head:subj:hd:arg:loc:cont:pron) *>
 c_cont:h_cons:[].


% Either the embedded NP is a comp noun or it is predicative already. (English 'he is a cert to win.')
% In the latter case the subject of the noun is identified with the subject of the predicative NP that is projected.
(pred_np,
 non_head_dtrs:hd:synsem:loc:cat:head:prd_noun) *>

(non_head_dtrs:hd:synsem:loc:cat:head:subj:Subj,
      synsem:loc:cat:head:subj:Subj).

