% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%       $RCSfile: copula.pl,v $
%%      $Revision: 243 $
%%          $Date: 2007-10-11 15:17:17 +0200 (Do, 11 Okt 2007) $
%%         Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%% $LastChangedBy: stefan $
%%        Purpose: macros for copula verbs.
%%       Language: Trale
%          System: TRALE 2.7.5 (release ) under Sicstus 3.12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.
:- multifile '*>'/2.
:- discontiguous '*>'/2.



% Diese Datei enthält Macros und Lexikoneinträge für die Kopula.

% I assume that the case of the predicational NP is identical to the case of the subject.
pred_np *>
 synsem:loc:cat:head:(case:morph_case:Case,
                      subj:hd:arg:loc:cat:head:case:morph_case:Case).

copula_verb :=
   (@predicational_copula_verb_le
   ;
    @specificational_copula_verb).

% The German, English, Maltese copula is treated as a raising verb
% with the raised element as least oblique and a predicative argument as the
% second oblique element. German allows arguments to be raised (optionally coherent)
% English is more strict and requires the predicative phrase to be fully saturated.
%
% Danish has a variant of predicational structures in which the clause initial element
% is the subject (as evidenced by the position of the negation) but nevertheless the predicate.
% This is account for by leaving the order of subj and predicate in the ARG-ST of the copula
% unspecified. The predicate can than be mapped to SPR.

specificational_copula_verb :=
  (@lexical_event_verb_type, % depends on the language, whether this is event_verb_root, event_verb_stem, or event_verb_word
%   @non_flip_verb, irrelevant for Danish
   @non_relational_le,
%  @scopal_subj_raising_verb_le, = raising_verb_le + 
   synsem:loc:(cat:(head:da:[],
%                     arg_st:append(raise(append(Subcat,Subj)),
%                                   [(@argument,
%                                     arg:loc:cat:(head:subj:(Subj,
%                                                             list_of_raised_arguments),
%                                                  subcat:Subcat))]),
                    % this is a more specific variant of the general copula.
                    % Adapted for Danish to make the grammar processable at all.
                    % Danish only allows predicates that are fully saturated and the subject
                    % consists of exactly one XP.
                    arg_st:(hd: (@argument,
                                 @n_str,  % since this is not the equational copula, and since it is Danish
                                          % the Subj of the predicate has to be a nominal with structural case.
                                 arg: @argument_synsem),
                            tl:tl:e_list, % speed = exactly two elements
                            append(raise(Subj),
                                  [(@argument,
                                    arg:loc:(cat:(head:(noun, % only nouns are allowed as subjects in specificational structures
                                                        subj:(Subj,
                                                             list_of_raised_arguments)),
                                                  subcat:list_of_spirits),
                                             cont:Cont))]))),
               cont:Cont)).



copula_verb(TeMo) :=
 (@irregular_verb(TeMo),
  dtrs:[@copula_verb]).

/*
copula_verb(VForm) :=
 (@irregular_verb(VForm),
  dtrs:[@copula_verb]).
*/


er_copula(Tense-tense) :=
  @copula_verb(Tense).

er(Tense-tense) :=
  @er_copula(Tense).

/*
er_copula(VForm) :=
  @copula_verb(VForm).

er(VForm) :=
  @sein_copula(VForm).

*/