% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: lexrules.pl,v $
%%  $Revision: 223 $
%%      $Date: 2007-09-12 19:27:44 +0200 (Wed, 12 Sep 2007) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: A German grammar using MRS semantics.
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile lex_rule/2.
:- discontiguous lex_rule/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.

:- lex_rule_depth(3).



head_specifier_phrase *>
  non_head_dtrs:hd:synsem:loc:cat:(head:dsl:none,
                                   spr:list_of_spirits,
                                   subcat:list_of_spirits).



% Does not work because of VP complements.
%head_argument_phrase *>
%  non_head_dtrs:hd:synsem: @argument_synsem.

head_argument_phrase *>
  non_head_dtrs:hd:synsem:loc:cat:subcat:list_of_spirits.

/*

full DSL value shared

verb_movement_lr :=
( @spr_saturated,
  synsem:(loc:(cat:(head:(verb,
                          subj:raise(Subj),
                          inv:plus),
                    subcat:hd:(@argument,
                               arg:loc:(cat:head:dsl:Loc,
                                        cont:gtop:GTop))), % LTOP comes from added relation
               cont:gtop:GTop),
          nonloc:Nonloc,
          trace:Trace,
          lex:Lex),
  h_cons:H_Cons,
  inf_marking:Zu,
  dtrs:[( word,
          synsem:(loc:(Loc,
                       cat:head:(verb,
                                 subj:Subj,
                                 inv:minus)),
                  nonloc:Nonloc,
                  trace:Trace,
                  lex:Lex),
          h_cons:H_Cons,
          inf_marking:Zu)]).

*/

verb_movement_lr :=
( @spr_saturated,
  synsem:(loc:(cat:(head:(verb,
                          subj:raise(Subj),
                          inv:plus,
                          stype:marked_stype),
                    xarg:Xarg,
                    subcat:hd:(@argument,
                               arg:loc:(cat:head:dsl:(%Loc,
                                                      cat:(head:Head,
                                                           spr:append(Pros,Spr),
                                                           subcat:Rest),
                                                      cont:Cont),
                                                           
                                        cont:gtop:GTop))), % LTOP comes from added relation
               cont:gtop:GTop),
          nonloc:Nonloc,
          trace:Trace,
          lex:Lex),
  h_cons:H_Cons,
  inf_marking:Zu,
  dtrs:[( word,
          synsem:(loc:(%Loc,
                       cat:(head:(Head,
                                  verb,
                                  subj:Subj,
                                  inv:minus),
                            xarg:Xarg,
                            spr:Spr,
                            subcat:append((list_of_pronouns,    % split the SUBCAT list into a list of pronouns and a rest
                                           Pros),Rest)),
                       cont:Cont),
                  nonloc:Nonloc,
                  trace:Trace,
                  lex:Lex),
          h_cons:H_Cons,
          inf_marking:Zu)]).



verb_initial_lr *>
( @verb_movement_lr,
  @scoping_lextop_lr,
  %complementizer_like_word,
  synsem:loc:cat:head:(vform:fin,
                       subj:[]),
  dtrs:[ synsem:loc:cat:(head:vform:fin,
                         subcat:list_of_non_raised_arguments) ]).


/*

Old constraint from an old version of the expletive LR. 28.09.2011

% This is a hack. It is used to prevent derived lexical items for the insertion of an expletive to
% undergo the verb movement LR. The problem is that it is impossible to distinguish between
% expletives that are  inserted in passives and expletives that are inserted for getting the
% positional constraints in wh clauses right.  09.09.2011 St. Mü.
verb_initial_lr *> synsem:nonloc:inher:slash:[].

*/

% Sätze mit Verb in Erststellung können Imperativsätze oder auch Interrogativsätze sein.
% Die beiden folgenden Sätze unterscheiden sich nur hinsichtlich ihrer Flexion:
%
%     Gib   Du mir das Buch!
%     Gibst Du mir das Buch?
%

% Konditionalsätze werden ignoriert.

(verb_initial_lr,
 synsem:loc:cat:subcat:[arg:nonloc:inher:slash:[]])      *> rels:hd:imperative_or_interrogative.

% Hier spielt das Element in SLASH eine entscheidende Rolle.
% Ist es eine Interrogativphrase, muß ein entsprechender Operator angenommen
% werden. Da die vorliegende Grammatik keine Interrogativpronomina enthält,
% wird die Fallunterscheidung nicht gemacht.

% Er gibt ihm das Buch.
% Jetzt gib mir schon das Buch!
%
% Wer kommt? Kann auch Interrogative sein. Das ist nur auf Satzebene zu entscheiden.
%
%(verb_initial_lr,
% synsem:loc:cat:subcat:[arg:nonloc:inher:slash:ne_list]) *> rels:hd:assertion_or_imperative.


/*
% We can infer the possible relation from morphology.
% This leaves only 'interrogative' for 'Lacht der Mann'.
(verb_initial_lr,
 rels:tl:hd:ind_conj) *> rels:hd:assertion_or_interrogative.

% imp in the type hierarchy that combines tense and mood.
% If the subject is dropped, the tense-mood type is forced down to imp.
% This also makes the clause type relation more specific.
(verb_initial_lr,
 rels:tl:hd:imp) *> rels:hd:imperative.

*/

verb_initial_lr ##
  Dtr
**>
( verb_initial_lr,
  dtrs:[Dtr])
morphs
  X becomes X.

/*

only one LR. This rule is incorporated into the verb movement LR given above.

pronoun_shift_lr *>
( @spr_saturated,
  synsem:(loc:(cat:(head:(verb,
                          subj:raise(Subj),
                          inv:plus),
                    subcat:hd:(@argument,
                               arg:loc:(cat:head:dsl:(%Loc,
                                                      cat:(head:Head,
                                                           spr:append(Pros,Spr),
                                                           subcat:Rest),
                                                      cont:Cont),
                                                           
                                        cont:gtop:GTop))), % LTOP comes from added relation
               cont:gtop:GTop),
          nonloc:Nonloc,
          trace:Trace,
          lex:Lex),
  h_cons:H_Cons,
  inf_marking:Zu,
  dtrs:[( word,
          synsem:(loc:(%Loc,
                       cat:(head:(Head,
                                  verb,
                                  subj:Subj,
                                  inv:minus),
                            spr:Spr,
                            subcat:shifted_pronouns(Pros,Rest)),
                       cont:Cont),
                  nonloc:Nonloc,
                  trace:Trace,
                  lex:Lex),
          h_cons:H_Cons,
          inf_marking:Zu)]).

*/


% unstressed pronouns have to be fronted before the negation
% stressed pronouns can be realized after the negation

fun list_of_pronouns(-,+,+).
fun list_of_pronouns(-).
list_of_pronouns(L) if
   when( (L=(e_list;ne_list)),
         undelayed_list_of_pronouns(L) ).

undelayed_list_of_pronouns([]) if true.
undelayed_list_of_pronouns([arg:(loc:cont:ppro,
                                  nonloc:inher:slash:[],                     % otherwise we could shift pronouns here and then extract them
                                  % if there is just one position for realizing the pronouns, then this position is also the one from which things get extracted.
                                  % but stressed pronouns can be realized after the verb and these can be extracted.
                                  lex:plus)|T]) if
   list_of_pronouns(T).


                 
/*
% copied from above, should be inherited
pronoun_shift_lr *>
( %@verb_movement_lr,
  @scoping_lextop_lr,
  %complementizer_like_word,
  synsem:loc:cat:head:(vform:fin,
                       subj:[]),
  dtrs:[ synsem:loc:cat:(head:vform:fin,
                         subcat:list_of_non_raised_arguments) ]).


pronoun_shift_lr ##
  Dtr
**>
( pronoun_shift_lr,
  dtrs:[Dtr])
morphs
  X becomes X.

*/


spr_subcat_verb_infl_lr := 
( %complex_word,
  synsem:(loc:cat:(head:(Head,
                         subj:[]),
                   xarg:XArg,
                   spr:(Subj,
                        tl:e_list), % Danish has to have a subject. (or rather an element in SPR)
                                    % This usully follows from what is in the valence list together
                                    % with what block_subject does. But in case of raising the category of the least
                                    % oblique element is unspecified and append also licecences empty Subj lists.

                                    % Danish differs from German in not restricting the category of the SPR
                                    % German uses the constraint block_subject which picks out the first NP with
                                    % structural case, but in Danish we have predicative NPs as the subject and these
                                    % NPs have lexical case. (See Müller, 1999 on German)
                   subcat:Subcat
                  ),
          nonloc:Nonloc,
          trace:Trace),
  inf_marking:IM,
  dtrs:[( stem,
          synsem:(loc:cat:(head:(Head,
                                 verb),
                           xarg:XArg,
                           arg_st:append(Subcat,Subj)),
                  nonloc:Nonloc,
                  trace:Trace),
          inf_marking:IM)]).


% This is like Maltese, except that Maltese copies Def and Marking under synsem:loc:cat from input to output.
% Maltese takes roots as input since root to stem mapping is not implemented yet.
fin_verb_infl_lr := 
( %complex_word,
  @spr_subcat_verb_infl_lr,
  @scoping_lextop_lr,
  synsem:loc:cat:head:vform:fin,
  dtrs:hd:synsem:loc:cat:arg_st:list_of_non_raised_arguments).


reg_fin_verb_infl_lr *>
( @fin_verb_infl_lr,
  @n_v_infl_lr,
  rels:hd:Tense,
  inf_marking:minus,
  affix:(fin_verb_i_suffix,
         tense:Tense)).


irreg_fin_verb_infl_lr *>
 (@fin_verb_infl_lr,
  phon:Phon,
  dtrs:[phon:Phon]).


fin_verb_infl_lr ##
( 
  Dtr
) 
 **>
( reg_fin_verb_infl_lr,
  affix:phon:[(a_ AffixPhon)],
  dtrs:[Dtr])
morphs
  X becomes (X,AffixPhon).


da_blocking_verb_infl_lr *>
 synsem:loc:cat:(spr:(Subj,
                      tl:e_list),
                 subcat:Subcat,
                 arg_st:split_arg_st_obl_subject(Subcat,Subj)).



fun split_arg_st_obl_subject(-,+,+).

%split_arg_st_obl_subject(ArgSt,Subcat,Subj) if !,true.  % debugging

%block_subject(Subcat,RestSubcat,Subj) if !. % debugging only
split_arg_st_obl_subject(ArgSt,Subcat,Subj) if
   when( ArgSt=(e_list;ne_list)
       , undelayed_split_arg_st_obl_subject(ArgSt,Subcat,Subj)).


% The first case is needed for intransitive verbs, that are passivized.
% These verbs do not have anything on ARG-ST. The mapping introduces an expletive,
% which is put on the SPR list.
undelayed_split_arg_st_obl_subject([],[],[@positional_np_expl]) if !,true.

% All other verbs do recursion till the end of the list.
undelayed_split_arg_st_obl_subject(ArgSt,Subcat,Subj) if split_arg_st_obl_subject_2(ArgSt,Subcat,Subj).


split_arg_st_obl_subject_2(ArgSt,Subcat,Subj) if
   when( ArgSt=(e_list;ne_list)
       , undelayed_split_arg_st_obl_subject_2(ArgSt,Subcat,Subj)).



% NP with or without structural case.
% The predicatve NP can be a subject as well.
% It has lexical case.
undelayed_split_arg_st_obl_subject_2([(SubjNP,
                                       @arg(loc:cat:head:noun))],[],[SubjNP]) if true.
 
% CP
% undelayed_split_arg_st_obl_subject_2([(SubjNP,
%                                      @n_str)],[],[SubjNP]) if true.

% NP with lexical case
% undelayed_split_arg_st_obl_subject_2([(Other,
%                                      @n_lex)],[Other],[@positional_np_expl]) if true.


undelayed_split_arg_st_obl_subject_2([(Other,
                                      @no_noun)],[Other],[@positional_np_expl]) if true.


undelayed_split_arg_st_obl_subject_2([X|T1],[X|T2],Subj) if
  when( T1 = (e_list;ne_list)
      , (T1=ne_list,
            split_arg_st_obl_subject_2(T1,T2,Subj))).


/*
undelayed_split_arg_st_obl_subject_2([X|T1],[X|T2],Subj) if
  when( T1 = (e_list;ne_list)
      , ( (T1=e_list,
           T2=e_list,
           X= @no_noun,
           Subj=[@positional_np_expl])
        ;
          split_arg_st_obl_subject_2(T1,T2,Subj))).

*/


% in embedded interrogatives we have the question phrase + SVO
%
% Que-Acc [Nom [Verb ...]
% Que-PP  [Nom [Verb ...]
%
% If the subject is questioned, an expletive `there' has to be used
% to fill the subject position.
%
% Que-Nom [there [Verb ...]
%


% not all languages need the inf-marking feature.
ppp_verb_infl_lr *>
  (%synsem:loc:cat:head:auxf:AuxF,
   inf_marking:minus).
%,
%   dtrs:hd:synsem:loc:cat:head:auxf:AuxF).

reg_ppp_verb_infl_lr *>
  (%ppp_verb_infl_lr,
   @n_v_infl_lr,
   affix:ppp_verb_i_suffix).


ppp_verb_infl_lr ##
   Dtr
  **>
( reg_ppp_verb_infl_lr,
  affix:phon:[(a_ Suff)],
  dtrs:[Dtr])
morphs
  X becomes (X,Suff).


% Maybe the ARG-ST mapping should not be done in the stem but in the inflectional rules.
% Right now it is done in the stems and then the inflactional rules may do a remapping.


subj_subcat_verb_infl_lr := 
( %complex_word,
  subj_subcat_mapping_le,
  @sem_identical_lr,
  synsem:(loc:cat:(head:(Head,
                         subj:tl:e_list), % Danish has to have a subject. (or rather an element in SPR)
                                    % This usully follows from what is in the valence list together
                                    % with what block_subject does. But in case of raising the category of the least
                                    % oblique element is unspecified and append also licecences empty Subj lists.

                                    % Danish differs from German in not restricting the category of the SPR
                                    % German uses the constraint block_subject which picks out the first NP with
                                    % structural case, but in Danish we have predicative NPs as the subject and these
                                    % NPs have lexical case. (See Müller, 1999 on German)
                   arg_st:ArgSt),
                  
          nonloc:Nonloc,
          trace:Trace),
  inf_marking:IM,
  dtrs:[( stem,
          synsem:(loc:cat:(head:(Head,
                                 verb),
                           arg_st:ArgSt),
                  nonloc:Nonloc,
                  trace:Trace),
          inf_marking:IM)]).


perf_verb_infl_lr *>
  (%synsem:loc:cat:head:auxf:AuxF,
   %subj_subcat_mapping_le,
   @subj_subcat_verb_infl_lr,
   synsem:loc:cat:head:vform:perf,
   inf_marking:minus).


reg_perf_verb_infl_lr *>
  (%perf_verb_infl_lr,
   @n_v_infl_lr,
   affix:ppp_verb_i_suffix).


perf_verb_infl_lr ##
   Dtr
  **>
( reg_perf_verb_infl_lr,
  affix:phon:[(a_ Suff)],
  dtrs:[Dtr])
morphs
   X becomes (X,Suff).






% % Nomina und Verben haben morphophonologische Merkmale,
% % die mit dem Affix übereinstimmen müssen.
% n_v_infl_lr :=
%   (infl_lr,
%    affix:(fk:FK,
%           morphophon:Morphophon),
%    dtrs:[infl:(fk:FK,
%                morphophon:Morphophon)]).

% Nomina und Verben haben morphophonologische Merkmale,
% die mit dem Affix übereinstimmen müssen.
n_v_infl_lr :=
   (infl_lr,
    affix:(fk:FK,
           morphophon:Morphophon),
    dtrs:[infl:(fk:FK,
                morphophon:Morphophon)]).


% n_v_infl_lr :=
%   (infl_lr,
%    affix:fk:FK,
%    dtrs:[infl:fk:FK]).




noun_infl_lr *>
(@n_v_infl_lr,
 @sem_identical_lr,
 synsem:(Synsem,
         loc:(cat:head:case:morph_case:Case,
              cont:ind:num:Num)),
 inf_marking:Zu,
 affix:(noun_i_suffix,
        num:Num,
        morph_case:Case),
 dtrs:[(synsem:(Synsem,
                loc:cat:head:noun),
        inf_marking:Zu,
        infl:noun_infl)]).




% If `Dtr' is placed before `infl' we get an instantiation error.
% This is defeinitely a Trale-Bug. 19.08.06
noun_infl_lr ##
  (infl:umlaut:UmlautStem,
   Dtr)
 **>
( noun_infl_lr,
  affix:(noun_i_suffix,                 % siehe oben
         num:Num,
         phon:[a_ Suff]),
  dtrs:[Dtr])
 if
   (%prolog(format("~w ~w~n", [Num,Suff])),
   get_irreg_n_stem(Num,UmlautStem,(a_ IStem)))
morphs
  X becomes (X,Suff) when IStem = none,
  X becomes (IStem,Suff).




% German only
attr_adj_infl_lr *>
(%infl_lr,
 synsem:loc:cat:head:mod:loc:cat:spr:[arg:loc:cat:head:(gen:Genus,
                                                        num:Num,
                                                        def:Def)],
 inf_marking:Zu,
 affix:(attr_adj_i_suffix,
        def:Def,
        num:Num,
        gen:Genus),
 dtrs:[inf_marking:Zu]).

% German and Persian differ from Maltese in the treatment of the SUBJ.
attr_adj_lr *>
 (synsem:loc:cat:spr:Spr,
  dtrs:hd:synsem:loc:cat:spr:Spr).



attr_adj_infl_lr ##
 Dtr
**>
(%@isect_attr_adj_or_participle_word,
 attr_adj_infl_lr,
 affix:phon:[(a_ Suffix)],
 dtrs:[Dtr])
morphs
  X becomes (X,Suffix).



prd_adj_infl_lr *>
(synsem:loc:cat:head:subj:[arg:loc:cont:ind:(gen:Genus,
                                             num:Num)],
 inf_marking:Zu,
 affix:(prd_adj_i_suffix,
        num:Num,
        gen:Genus),
 dtrs:[inf_marking:Zu]
 ).


prd_adj_infl_lr ##
  Dtr
 **>
( prd_adj_infl_lr,
  affix:phon:[(a_ Suffix)],
  dtrs:[Dtr])
morphs
  X becomes (X,Suffix).



% definiteness marking

definiteness_lr *>
 (@cont_identical_lr,
  %complex_word,
  synsem:(loc:cat:(head:Head,
                   spr:[], %list_of_spirits, should be the spirit, but we do not need it ...
                   subcat:Subcat),
          nonloc:Nonloc,
          trace:Trace),
  rels:[(Quant,
         def_q,
         arg0:Ind,
         lbl:QHandle,
         rstr:Restr)|Rels],
  h_cons:[(qeq,
           harg:GTop,
           larg:QHandle),
          (qeq,
           harg:Restr,
           larg:LTop)|HCons],
  subtract_rels:SR,
  dtrs:[(word,
         synsem:(loc:(cat:(head:(noun,
                                 Head),
                           spr:[(arg:loc:cat:spr:[], % This is needed for technical reasons, otherwise residues of the constraint (list_of_spirits) remain.
                                 realized:minus)],
                           subcat:Subcat),
                      cont:(ind:Ind,
                            gtop:GTop,
                            ltop:LTop, 
                            quant:Quant)),
                 nonloc:Nonloc,
                 trace:Trace),
         rels:Rels,
         h_cons:HCons,
         subtract_rels:SR)]).


definiteness_lr ##
 (Dtr,
  synsem:loc:cont:ind:(num:sg,
                       gen:common))
**>
(%@isect_attr_adj_or_participle_word,
 definiteness_lr,
 dtrs:[Dtr])
morphs
  X becomes (X,en).

definiteness_lr ##
 (Dtr,
  synsem:loc:cont:ind:(num:sg,
                       gen:neu))
**>
(%@isect_attr_adj_or_participle_word,
 definiteness_lr,
 dtrs:[Dtr])
morphs
  X becomes (X,et).

definiteness_lr ##
 (Dtr,
  synsem:loc:cont:ind:num:pl)
**>
(%@isect_attr_adj_or_participle_word,
 definiteness_lr,
 dtrs:[Dtr])
morphs
  X becomes (X,ne).


% Danish: the subject is extracted
% The subject is extracted. (The second from last element)
% The LOCAL values have to be identified by a trace rather than beeing
% identical by chance.

positional_expl_lr *>
 synsem:loc:cat:arg_st:append(_,[arg:(loc:(Loc,
                                           cont:ind:ref),
                                      nonloc:inher:slash:[Loc]),_]).



get_irreg_n_stem(pl,(a_ UmlautStem),  (a_ UmlautStem))  if true.
get_irreg_n_stem(sg,_,_) if true.
