% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: german-macros.pl,v $
%%  $Revision: 316 $
%%      $Date: 2008-06-12 17:21:16 +0200 (Do, 12 Jun 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: German grammar using MRS
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.10.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.
:- multifile '*>'/2.
:- discontiguous '*>'/2.




% words that do not undergo inflection do not take an `at'. AT is a technical feature.
% used for infinitive inflection.
simple_word *> inf_marking:minus.


determiner(Numerus) :=
 synsem:loc:cat:head:num:Numerus.

determiner(Numerus,Genus,Def) :=
(%determiner_word,
 @determiner(Numerus),
 synsem:loc:cat:head:(gen:Genus,
                      def:Def)).


% empty determiner and overt determiners without gender
det_nq(Numerus,Quant) :=
 (@determiner(Numerus),
  rels:[Quant]).


% determiner with gender 
det(Numerus,Genus,Def,Quant) :=
(@determiner(Numerus,Genus,Def),
 @overt_determiner_word,
 @det_nq(Numerus,Quant)).

% determiner without gender 
det(Numerus,Def,Quant) :=
(@determiner(Numerus,genus,Def),
 @overt_determiner_word,
 @det_nq(Numerus,Quant)).


% der
positional_np_expl :=
  @np_expl_lex.

% German takes dative, Danish takes accusative
ditrans_verb_le *>
  synsem:loc:cat:subcat:hd: @np(acc,_).

lexical_verb_type :=
  verb_stem.

% Do we need this?
lexical_event_verb_type :=
  verb_stem.

% all lexical verbs are univerted. Only after applying a lexical rule we get the inverted verbs.
verb_le *>
 synsem:loc:cat:head:inv:minus.

% used to be a constraint on listemes, but now also applies to newly created stems.
% This is needed for the insertion of expletives, which is done by a lexical rule.
%verb_le *>
(stem,
 synsem:loc:cat:head:verb) *>
 (synsem:loc:cat:(head:subj:[],
                  xarg:Subj,
                  spr:Subj,
                  subcat:Subcat,
                  arg_st:split_arg_st_obl_subject(Subcat,Subj))).


irregular_verb(TeMo) :=
 (irreg_fin_verb_infl_lr,
%  synsem:loc:cat:subcat:subj_verb_agreement((per:Per,num:Num)),
  rels:hd:TeMo).



passive_aux(TeMo) :=
 (@irregular_verb(TeMo),
  dtrs:[@agentive_passive_aux_verb]).

% The passive auxiliary takes a VP (SUBCAT saturated, SPR non-empty) and
% raises the SPR to its own ARG-ST. From there it is mapped to its specifier.
% passive_aux :=
%  (@lexical_verb_type,
%   synsem:loc:cat:head:da:[],                      %% DA-Wert schließt Passivierung von Hilfsverben aus.
%   @non_relational_le,
%   @non_idiomatic_le,
%   synsem:loc:(cat:arg_st:[arg:loc:(cat:(head:vform:ppp,
%                                         spr:[Spr],
%                                         subcat:list_of_spirits),
%                                    cont:Cont),
%                           Spr],
%               cont:Cont)).


have_perfect_aux(TeMo) :=
 (@irregular_verb(TeMo),
  dtrs:[@perf_perfect_aux_verb]).


ppp_inf_verb :=
  (@raising_verb,
   @verbal_complement_le,
   synsem:loc:cat:arg_st:hd:arg:loc:cat:head:vform:ppp).


v_modifier  :=
 (@modifier,
  synsem:loc:cat:head:(%pre_modifier:plus,
                       mod:loc:cat:(head:verb,
                                    spr:(ne_list,           % important for post-VP modifieres,
                                                            % otherwise we would get spurious ambiguities.
                                         list_of_unrealized_args), % In a spirits approach we have to ensure that things are actually unrealized 11.01.2012
                                    subcat:list_of_spirits))).


% negation
scopal_pre_modifier_adv :=
 (@scopal_adverb,
  synsem:loc:cat:head:pre_modifier:plus).

% 
% scopal_pre_modifier_adv(Reln) :=
%  (@scopal_adv(Reln),
%   synsem:loc:cat:head:pre_modifier:plus).

% Danish and German, but Persian does not have head movement.
complementizer_like_word *>
  synsem:loc:cat:subcat:hd:arg:trace:minus_or_head_movement.

/*
% 
bjarnes_complementizer_word :=
 (@spr_saturated,
  @non_idiomatic_le,
  simple_word,
  @overt_le,
  @non_rel_le,
  @non_relativizer_le,
  @simple_le,
  @ltop_lbl_le,
  @arg0_le,

   
  synsem:loc:cat:(head:(comp,        %dsl:none,  German only
                        initial:plus),
                  subcat:[  (@argument_xp,
                             arg:( loc:(cat:(head:(verb,
                                                   vform:fin),
                                                   %initial:minus\), German and Persian, but not Danish
                                             subcat:list_of_spirits),
                                        cont:ltop:SCompLTop),
                                   trace:minus)),
                            (@argument_xp,
                            arg:( loc:(cat:head:(adv,
                                                 mod:loc:cont:ltop:VerumLbl),
                                       cont:ltop:AdvLTop),
                                trace:minus)) ]),
  rels:[
        (pred:(a_ if),
         lbl:LblIf,
         arg3:AdvLTop),
        (verum_rel,
         lbl:VerumLbl,
         arg3:VerumArg)],
  h_cons:[% complementizer scopes over adverb:
          (qeq,
           harg:IfLTop,
           larg:AdvLTop),
          % The adverb scopes over verum_rel since the ltop of the modifiee is identified with
          % the label of the verum_rel

          % verum_rel scopes over the sentential argument of the complementizer
          (qeq,
           harg:VerumArg,
           larg:SCompLTop)
          ]).

*/

% usually the noun is the head. Since the negated NP has to 
% negated_determiner :=
%  (@scopal_v_modifier_word,
%   synsem:loc:cat:(head:noun,
%                   subcat:[arg:(@n_bar(Ind),
%                                loc:cont:(gtop:GTop,
%                                          ltop:LTop))]),
%                      
%   rels:[(pred: (a_ neg_rel),
%          arg3:NegArg),
%         (one_q,
%          lbl:QuantLBL,
%          arg0:Ind,
%          restr:   )],
%   h_cons:[(qeq,
%            harg:NegArg,
%            larg:QuantLBL)])


% This is the description of unnegated determiners.
% The negated determiner has the negation relation in addition.
% The new local TOP is the LABEL of the negation.
% determiner_le *>
%  (@determiner_or_possp_le,
%   @relational_arg0_le,
%   synsem:loc:(cat:head:spec:loc:cont:(ind:Ind,
%                                       gtop:GTop,
%                                       ltop:Noun_LTop),
%               cont:(gtop:GTop,
%                     ltop:Noun_LTop)), % This differs from standard MRS, where the LTOP of a quantifier is
%                                       % not instantiated. See the Chinese grammar.
%    rels:hd:arg0:Ind,
%    h_cons:tl:tl:e_list).

negated_determiner :=
 (@overt_word,
  simple_word,
  @determiner_or_possp_le,
  @relational_arg0_le,
  @non_rel_le,
  @non_que_le,
  synsem:loc:(cat:head:spec:loc:cont:(ind:Ind,
                                      gtop:GTop,
                                      ltop:Noun_LTop),
              cont:(gtop:GTop,
                    ltop:Noun_LTop)),     
  rels:[(one_q,
         lbl:QuantLbl,
         arg0:Ind),
        (negation_rel,
         %lbl:NegLbl,
         arg3:NegHandle)],
  h_cons:tl:tl:[(qeq,
                 harg:NegHandle,
                 larg:QuantLbl)]).


% coordination
dsl_none :=
  dsl:none.

% not used
non_flip_verb := sign.

% not used
cluster_minus_argument := argument.


% not used
cluster_minus_synsem :=
  synsem.



% not used
definite_le :=
 sign.


% not used
unmarked := sign.

% not used
incoherent_le := sign.




% in contrast to Persian there are no resumptive pronouns in Danish.
pronoun_le *> synsem:nonloc:inher:slash:[].
