% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: rules.pl,v $
%%  $Revision: 316 $
%%      $Date: 2008-06-12 17:21:16 +0200 (Do, 12 Jun 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Eine kleine Spielzeuggrammatik für die Lehre
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- multifile rule/2.


% This file may be ignored. It guides the parser and is needed for technical reasons.

h_arg ## (head_argument_phrase,
          @head_initial,                                  % nur für Regelausgabe
             synsem:loc:cat:head:initial:plus,
             head_dtr:HeadDtr,
             non_head_dtrs:[(NonHeadDtr,
                             % VM-Trace kann nie allein hier stehen, immer ein Argument, oder eingebettetes Verb
                             % wird für mehrfache Vorfeldbesetzung gebraucht, sonst Nichterminierung
                             synsem: trace:extraction_or_minus)
                           ])
  ===>
cat> HeadDtr,
cat> NonHeadDtr.


arg_h ## (head_argument_phrase,
             synsem:loc:cat:head:initial:minus,
             head_dtr:HeadDtr,
             non_head_dtrs:[(NonHeadDtr,
                             synsem: (@argument_synsem,
                                      loc:cat:head:dsl:none,
                                      trace:extraction_or_minus))]
         )
  ===>
cat> NonHeadDtr,
cat> HeadDtr.


h_adj ## (head_adjunct_phrase,
             head_dtr:HeadDtr,
             non_head_dtrs:[(NonHeadDtr,
                             synsem:loc:cat:head:pre_modifier:minus)])
  ===>
cat> HeadDtr,
cat> NonHeadDtr.


adj_h ## (head_adjunct_phrase,
             head_dtr:HeadDtr,
             non_head_dtrs:[(NonHeadDtr,
                             synsem:loc:cat:head:pre_modifier:plus)])
  ===>
cat> NonHeadDtr,
cat> HeadDtr.


spr_h ## (head_specifier_phrase,
             head_dtr:HeadDtr,
             non_head_dtrs:[NonHeadDtr])
  ===>
cat> NonHeadDtr,
cat> HeadDtr.

% % for a postverbal subject in V2 clauses
% h_spr ## (head_specifier_phrase,
% %             head_dtr:HeadDtr,
%              head_dtr:(HeadDtr,
%                        synsem:(loc:cat:head:(verb,
%                                              inv:plus),
%                                lex:plus)), % The subject has to be combined with the verb directly,
%                                            % otherwise we would admit:
%                                            % Læste en bog Bjarne.     St. Mü. 28.06.2009
%              non_head_dtrs:[(NonHeadDtr,
%                              synsem:(
%                                      trace:minus         % speed: steht eigentlich im Lexikon
%                                     ))])
%   ===>
% cat> HeadDtr,
% cat> NonHeadDtr.


v2_cl ## (v2_clause,
            @head_final,                                  % nur für Regelausgabe
             head_dtr:HeadDtr,
             non_head_dtrs:[NonHeadDtr])
  ===>
cat> NonHeadDtr,
cat> HeadDtr.

int_cl ## (interrogative_clause,
            @head_final,                                  % nur für Regelausgabe
             head_dtr:HeadDtr,
             non_head_dtrs:[NonHeadDtr])
  ===>
cat> NonHeadDtr,
cat> HeadDtr.


% rc ## (rc,
%        non_head_dtrs:[Filler,Clause])
%   ===>
% cat> Filler,
% cat> Clause.



% Hierfür würde man eigentlich eine Lexikonregel verwenden,
% die den Verbstamm auf den zu-Infinitiv abbildet, aber
% das geht aus technischen Gründen nicht, da zwischen
% `zu' und Infinitiv ein Leerzeichen steht.
% Das ist übrigens bei den Partikelverben nicht so ...

% der zu lesende Aufsatz, eigentlich `zu lesen' + de
% Der Aufsatz ist zu lesen.
inf_at ##

 (complex_word,
  synsem:Synsem,
  rels:Rels,
  subtract_rels:SR,
  h_cons:HCons,
  inf_marking:minus)
  ===>
  % Als erste Tochter nehmen wir irgendein `zu', das in der Grammatik vorhanden ist.
  % Es muß eindeutig sein!
cat> (word,
      phon:[(a_ at)]),
cat> (word,
      synsem:Synsem,
      rels:Rels,
      subtract_rels:SR,
      h_cons:HCons,
      inf_marking:plus).



preposed_neg ## (preposed_negation_phrase,
             %non_head_dtrs:NonHeadDtrs,
              non_head_dtrs:[Comp,Neg,S])
  ===>
cat> Comp,
cat> Neg,
cat> S.

