% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: speed.pl,v $
%%  $Revision: 298 $
%%      $Date: 2008-03-11 20:44:55 +0100 (Di, 11 Mär 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: Linguistisch nicht signifikante Hacks
%%             zur Effizienzsteigerung beim Bottom-Up-Parsen
%%   Language: Trale
%      System: TRALE 2.7.5 under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.

:- multifile macro/2.
:- discontiguous macro/2.



% Die folgenden Regeln schließen die Erzeugung von Phrasenstrukturregeln aus,
% die in der Grammatik nicht gebraucht werden.

head_filler_phrase *>
   non_head_dtrs:hd:phon:ne_list.  % keine Spuren (TRACE-) bzw. leere Elemente (Determinator).
                                     % Evtl. bei Vorfeldellipse ändern.





% Bedingung, die in rules.pl für kopffinale Regel verwendet wird.
argument_synsem :=
 (loc:cat:(head:(% Linguistisch ist es nicht notwendig, aber TRALE rechnet
                 % die leeren Elemente aus und hängt sich dabei auf, wenn
                 % man nicht Spuren per Hand ausschließt.
                 % dsl:none,
                 
                 mod:none, % keine Adjunkte: Adjektive ^ Adverbien
                 spec:none % keine Determinierer
                ),
           spr:list_of_spirits, 
           subcat:list_of_spirits     % Das ist nicht nur eine speed-Beschränkung. Steht jetzt auch in syntax.pl
                                      % argument_synsem wird aber auch unten in speed.pl noch genutzt.
                                      % Diese Beschränkung sorgt dafür, dass inkohärente Konstruktionen wirklich
                                      % eine Maximalprojektion verlangen.
          )).



% The following excludes the combination of a head movement trace with an incomplete intransitive verb.
% St. Mü. 24.03.2011

% Reduced active edges:

% 2 min 13.670 sec CPU time.        No solution; 188 passive edges
% 1 min 36.730 sec CPU time.        No solution; 157 passive edges

(headed_phrase,
 head_dtr:hm_trace) *> non_head_dtrs:hd:synsem:loc:cat:spr:list_of_spirits.

% ebenfalls linguistisch nicht notwendig, verringert
% aber die Menge der von TRALE berechneten Regeln.

% ohne diese Bedingung könnte die Verbspur beliebig oft
% als Adjunkt auftreten.

head_adjunct_phrase *>
  non_head_dtrs:hd:synsem:loc:cat:head:dsl:none.

% allgemeiner
%head_non_argument_phrase *>
%  (non_head_dtrs:[synsem:loc:cat:head:dsl:none]).










% Achtung: Oft gibt er nicht dem Mann das Buch.

% er lacht oft.
% ist sonst mehrdeutig.

% * [ oft [ _ _ ]]
%   [ _   [ oft _ ]]

% This rules out:
% Vinderen_i er_j Max ikke _j _i.
%(head_adjunct_phrase,
% head_dtr:head_argument_phrase) *> head_dtr:non_head_dtrs:hd:synsem:trace:minus.



% Relativsätze

% Information über Relativpronomina kann nur aus der vorangestellten Relativphrase kommen.
% Die Relativphrase kann aber nur aus dem Relativpronomen selbst oder aus einer PP bestehen.

head_adjunct_phrase *>
  non_head_dtrs:hd:synsem:nonloc:inher:rel:[].

% Das stimmt nicht für zu-Infinitive wie in
% das Buch, das zu lesen ich ihm empfohlen habe
% Deshalb wird diese Beschränkung später revidiert.
(headed_phrase,
 synsem:loc:cat:head:verb) *> synsem:nonloc:inher:rel:[].








/*
% Für die Anzeige der Regeln
head_argument_phrase *> (head_dtr:HD,
                            non_head_dtrs:[NHD],
                            ( head_dtr:synsem:loc:cat:head:initial:plus,
                              dtrs:[HD,NHD]
                            ; head_dtr:synsem:loc:cat:head:initial:minus,
                              dtrs:[NHD,HD]
                            )).

*/






    % Diese Beschränkung für die SUBJ-Liste ist nicht unbedingt nötig, da sie aus der
    % Anwendung von block_subject folgt. Allerdings führt die Kombination
    % von Hilfsverben mit Extraktionsspuren bei der Kombination mit
    % einem Komplementierer, der eine vollständige Verbalprojektion verlangt,
    % zu instantiierten SUBCAT-Listen, weshalb append dann die Zerlegung
    % der Listen vornimmt. Dann entstehen ohne die Beschränkung zweistellige
    % SUBJ-Listen.
    % daß er das buch wird lesen müssen: 318 statt 402 Kanten

verb *> subj:list_with_zero_or_one_element.




% The specifier is the last element that is combined with its head.
% Without this constraint the empty verb combines with a subject NP and since
% the SUBCAT list is not specified, it can be combined with an extraction trace
% in addition. The constraints are delayed, so nobody finds out that this is implossible.
% St. Mü. 10.07.2009
head_specifier_phrase *> synsem:max_:plus.


head_argument_phrase *> head_dtr:synsem:max_:minus.


% This makes the rule computation terminate. It excludes head traces and combinations
% of it with adjuncts as arguments.
% St. Mü. 10.07.2009
(head_argument_phrase,
 non_head_dtrs:hd:synsem:loc:cat:head:dsl:local) *> phon:ne_list.

(head_argument_phrase,
 synsem:loc:cat:head:dsl:local) *> non_head_dtrs:hd:synsem:loc:cat:head:dsl:none.



% I assume that adjuncts are VP adjuncts.
% This saves two edges for every full NP, since the NP projects to an S with an empty verb,
% which then gets modified by an empty adjunct. St. Mü. 10.07.2009

% reduces the number of edges from 379 to 305 (3 min 40.730 to 2 min 40.980)
% Peter arbejder, hvis han ikke læser en bog.

(head_adjunct_phrase,
 synsem:loc:cat:(head:verb,
                 spr:hd:realized:plus))
% goal list_of_spirits(Spr) impossible to specify relational constraint in the antecedent

*> non_head_dtrs:hd:synsem:trace:minus.

% specifiers are not adjuncts
% 305 -> 289 edges  (2 min 40.980 to 2 min 18.240)
% head_specifier_phrase *> non_head_dtrs:hd:synsem:loc:cat:head:mod:none.

% This is even more restrictive. (and probably wrong, since there a subject sentences)
% 305 -> 230 edges (2 min 40.980 to 1 min 42.690 sec)
(head_specifier_phrase,
 synsem:loc:cat:head:verb) *> non_head_dtrs:hd:synsem:loc:cat:head:noun.


% Adjuncts attach to VPs. Due to the empty verbal head we do not know
% the valence requirements of the verb. The constraints coming from the adjunct are delayed.
% The result of this is that a modified empty verbal head (or a projection thereof) can be combined
% with further arguments. Such structures are never licenced in actual analyses.

% 230 -> 193 (1 min 42.690 sec -> 1 min 21.740 sec)
(head_adjunct_phrase,
 synsem:loc:cat:head:verb) *> synsem:max_:plus.


head_argument_phrase *> head_dtr:synsem:max_:minus.


% German has non-finite verb traces, since there are multiple frontings
% but these do not exist in Danish.
hm_trace *>
  synsem:loc:cat:head:vform:fin.


% This excludes the combination of non-finite VPs with the subject.
% This is linguisticially not correct, but the projections are currently not needed.

(head_specifier_phrase,
 synsem:loc:cat:head:verb) *> synsem:loc:cat:head:vform:fin.

