% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: german-syntax.pl,v $
%%  $Revision: 316 $
%%      $Date: 2008-06-12 17:21:16 +0200 (Do, 12 Jun 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: A grammar for German
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.10.1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.
:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.

non_dsl_head *> dsl:none.

verb    *> initial:plus.
noun    *> initial:plus.

% This is lexeme specific.
%adv     *> pre_modifier:plus.

adj     *> initial:minus.
mod_adj *> pre_modifier:plus.

mod_comp *> pre_modifier:minus.



prd_adj *> da:[].          % adjectives are unaccusative (Haider)

% This is different from German. In order to allow adjuncts in the verbal complex,
% the lex value is not instantiated in head_adjunct_phrase for German.
% In Danish, the head_specifier_phrase refers to LEX, hence adjunct + VP has to be LEX-
head_adjunct_phrase  *> synsem:lex:minus.

% head filler phrase has subtypes for V2 (inverted) and interrogative (uninverted) clauses.
head_filler_phrase *>
   head_dtr:synsem:loc:cat:%(head:inv:plus,
                                 % initial:plus), not needed since all verbs are initial:plus
                                 % and the head has to be a verb. St. Mü. 18.03.2011
                            spr:list_of_spirits.

% Danish, German, Chinese, Maltese, but not Persian
headed_struc *>
   synsem:nonloc:inher:slash:list_with_zero_or_one_element.


% Teil der Verbbewegungsanalyse

% * Er schläft schläft.
%
(headed_phrase,
 head_dtr:((complex_word;
            simple_word),
           % word, % should be sufficient, seems to be a TRALE bug.
           phon:ne_list)) *> head_dtr:synsem:loc:cat:head:dsl:none.

% Die obige Implikation schließt auch infinte Verben mit gefülltem
% DSL-Wert aus. Leider nehmen aber nicht alle infiniten Verben Argumente
% oder Adjunkte zu sich, weshalb `Wird geregnet' lizenziert wird, und
% zwar mit der absurden Struktur in der `wird' eine Verbspur verlangt,
% aber mit dem nicht-finiten Verb kombiniert wird.
% Man kann nicht für alle nicht-finiten Verben einfach sagen, daß
% DSL none ist, da dann die mehrfache Vorfeldbesetzung nicht mehr funktionieren
% würde.

(headed_phrase,
 non_head_dtrs:hd:(complex_word,
                                % word, % should be sufficient, seems to be a TRALE bug.
                   phon:ne_list)) *> non_head_dtrs:hd:synsem:loc:cat:head:dsl:none.





fun list_of_at_minus(-).
list_of_at_minus(L) if
   when( (L=(e_list;ne_list)),
         list_of_at_minus2(L) ).


list_of_at_minus2([]) if true.
list_of_at_minus2([H|T]) if
   when( (H=(word;phrase)),
         undelayed_list_of_at_minus([H|T]) ).

undelayed_list_of_at_minus([]) if true.
undelayed_list_of_at_minus([(word,
                             inf_marking:minus)|T]) if
   list_of_at_minus(T).
undelayed_list_of_at_minus([phrase|T]) if
   list_of_at_minus(T).

% Sorgt dafür, daß Objekte, die noch ein `at' benötigen,
% nicht in der Syntax als Töchter auftreten können.
% Das kann nicht ausschließlich an Typen festgemacht werden,
% da `at lesende' eine Form mit Adjektivflexion ist. Flektierte
% Objekte sind aber normalerweise syntaktische Objekte (z.B. `kluge') 
phrase *> dtrs:list_of_at_minus.

(head_specifier_phrase,
 dtrs:tl:hd:synsem:loc:cat:head:verb) *> synsem:loc:cat:head:inv:minus.
  


% This should inherit from headed structure. This would do the percolation
% of the head features, of the semantics.
% However the nonlocal feature principle subtracts the to_bind of the head daughter
% and this value is [].
preposed_negation_phrase *>
 ( synsem:(loc:cat:(head:Head,         % HFP
                    subcat:[(arg:S,
                             realized:plus)]),
           nonloc:inher:(rel:[],
                         que:[],
                         slash:[])),
   c_cont:( hook:ind:CompInd,  %_Hook, % HFP  % The global top is different.
%                  (ind:CompInd,
%                   %ltop:CompLTop,
%                   lextop:CompLexTop),
            rels:[(verum_rel,
                   lbl:VerumLbl,
                   arg3:VerumArg)], 
            h_cons:[% The complementizer outscopes negation.
                    (qeq,
                     harg:CompArg,
                     larg:NegationLbl),
                    % negation outscopes verum_rel
                     (qeq,
                      harg:NegationScope,
                      larg:VerumLbl
                      ),
                    % verum outscopes all quantifiers in the clause
                    % since the quantifiers outscope the predicates in the clause, we get the right scopings.
                      (qeq,
                       harg:VerumArg,
                       larg:SGTop 
                       )
%                    ,
%                     % verum outscopes the clause without negation
%                      (qeq,
%                       harg:VerumArg,
%                       larg:ClauseLbl
%                       )
                   
                   ]),
  non_head_dtrs:[ (synsem:(loc:(cat:(head:(Head,
                                           comp),
                                     subcat:[(arg:S,
                                              realized:minus)]),
                                cont:
                                %%      Hook,
                                     ind:CompInd,
                               idiomatic:minus),
                           nonloc:inher:slash:[]),
                                % We could use Key to access this argument, but since we are constructional anyway,
                   % we can have a look at RELS as well.
                      rels:hd:arg1:CompArg),
                 (              % adv or negated NP
                  synsem:loc:NegP,
                  rels:(hd:(negation_rel,
                            lbl:NegationLbl,
                            arg3:NegationScope)
                       ;tl:hd:(negation_rel,
                               lbl:NegationLbl,
                               arg3:NegationScope))),
                 (synsem:(S,
                          loc:cont:(key:lbl:ClauseLbl,
                                    gtop:SGTop),      % This ensures that Verum outscopes all quantifiers in the clause.
                          nonloc:inher:slash:[NegP]))]).
                                  

% if the head-dtr is final, the non_head_dtr has to be a negated NP.
HeadDtr^(head_argument_phrase,
         head_dtr:HeadDtr,
         dtrs:tl:hd:HeadDtr) *> non_head_dtrs:hd:(synsem:loc:cat:head:noun,
                                                  rels:tl:hd:negation_rel).

% if the head-dtr is initial, the non_head_dtr may not be negated.
HeadDtr^(head_argument_phrase,
         head_dtr:HeadDtr,
         non_head_dtrs:hd:synsem:loc:cat:head:noun,
         dtrs:hd:HeadDtr) *> non_head_dtrs:hd:rels:(e_list  % pronoun, trace, ....
                                                   ;tl:hd: @not_(negation_rel)).



/*

% This was a first attempt. 07.03.2011

% there_subj_extraction_phrase := phrase & compositional_sw_or_phrase.



there_subj_extraction_phrase *>
(synsem:(loc:(cat:(head:Head,
                   spr:list_of_spirits,
                   subcat:Subcat),
              cont:Cont),
         nonloc:(inher:(rel:Rel,
                        que:Que,
                        slash:[Loc]),
                 to_bind:TB)),
 non_head_dtrs:[synsem:(loc:(cat:(head:(Head,
                                        vform:fin,
                                        dsl:none,
                                        inv:minus),
                                  spr:[arg:loc:(Loc,
                                                cont:ind:ref)],
                                  subcat:(Subcat,
                                          list_of_spirits)),
                             cont:Cont),
                        nonloc:(inher:(rel:Rel,
                                       que:Que,
                                       slash:[]),
                                to_bind:TB)),
                 (phon:[a_ der], % also excludes traces
                     synsem:loc:(cat:head:noun,
                                 cont:ind:expl))]).

*/

/*

% there_subj_extraction_phrase := headed_binary_phrase & head_non_adjunct_non_specifier_struc & head_non_argument_non_cluster_phrase  & head_non_filler_struc & head_non_clld_struc & headed_compositional_sw_or_phrase.

there_subj_extraction_phrase *>
(synsem:loc:cat:marking:marked,
 head_dtr:synsem:loc:cat:(head:(vform:fin,
                                dsl:none,
                                inv:minus),
                          spr:(list_of_spirits,
                               [arg:nonloc:inher:slash:[(cat:head:case:morph_case:nom,
                                                         cont:ind:ref)]]),
                          subcat:list_of_spirits,
                          marking:unmarked),
 non_head_dtrs:[(phon:[a_ der], % also excludes traces
                 synsem:loc:(cat:head:noun,
                             cont:ind:expl))]).

% And what about coordination?
% I wonder who [ [_ comes] and [_ sings] ]
%
% St. Mü. 15.07.2011

*/   


interrogative_clause *>
% head_dtr:non_head_dtrs:[synsem:nonloc:inher:slash:[]].
  head_dtr:synsem:loc:cat:xarg:hd:arg:nonloc:inher:slash:[].

% The following constraint holds both for V2 clauses, but not for interrogatives:
%
% * Bjarne arbejder der.
%
%   Bjarne spørger hvem der læser bogen.  ;; 'Bjarne asks who EXPL reads book.DEF' "Bjarne asks who reads the book."
%
% This is basically a ban of string-vacuous movement, which would arise in interrogatives but not in V2 clauses, in
% which we have the verb inbetween the extraction position and the prefield. This should be formulated in a more insightful way.
% St. Mü. 07.01.2012
v2_clause *>
  head_dtr:synsem:loc:cat:xarg:hd:arg:loc:cont:ind: @not_(expl). % ref excludes the event type of specificational SPRs

v2_clause *>
  synsem:v2:plus.
 



root :=
 (synsem:(loc:cat:(head:dsl:none,
                   spr:list_of_spirits,
                   subcat:list_of_spirits),
          nonloc:inher:slash:[]),
  subtract_rels:[]).

initial_fin_verb :=
 (@root,
  synsem:loc:cat:head:(verb,
                       inv:plus,
                       vform:fin)).

interrog :=
 @initial_fin_verb.

decl :=
 (@initial_fin_verb,
  synsem:v2:plus).

imp :=
  @initial_fin_verb.

