% -*-trale-prolog-*-
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   $RCSfile: participle-lexrules.pl,v $
%%  $Revision: 280 $
%%      $Date: 2008-01-11 15:00:27 +0100 (Fr, 11 Jan 2008) $
%%     Author: Stefan Mueller (Stefan.Mueller@cl.uni-bremen.de)
%%    Purpose: A German grammar using MRS semantics.
%%   Language: Trale
%      System: TRALE 2.7.5 (release ) under Sicstus 3.12.0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- multifile if/2.
:- discontiguous if/2.
:- multifile fun/1.
:- discontiguous fun/1.
:- multifile ':='/2.
:- discontiguous ':='/2.
:- multifile lex_rule/2.
:- discontiguous lex_rule/2.
:- multifile '*>'/2.
:- discontiguous '*>'/2.


adj_derivation_lr *>
 (zu:Zu,
  dtrs:[ zu:Zu ]).


% The participle can be used predicatively in Persian (together with šodan).
% The lexical rule derives the predicative form. The attributive form is derived
% by another lexical rule.
% For German we licence a stem. In principle both attributive and predicative forms
% could be derived from this stem, however the predicative form is not attested,
% so the stem is restricted to have a head value of mod_adj, which excludes the predicative
% usage.
adj_derivation_lr *>
  synsem:loc:cat:head:mod_adj.                 % kann nicht prädikativ genutzt werden,
                                               % dafür sollte es eine prinzipiellere Erklärung geben.



% * der zu gelingende Aufsatz
% der zu lesende Aufsatz
inf_part_derivation_lr *>
 (%part_derivation_lr,
  @scoping_lr,
  rels:hd:pred: (a_ modal_rel),
  dtrs:[synsem:loc:cat:head:vform:inf]).


inf_part_derivation_lr ##
  Dtr
 **>
( inf_part_derivation_lr,
  dtrs:[Dtr]
  )
morphs
  X becomes (X,d).

% lesen -> lesend
bse_part_derivation_lr *>
(@sem_identical_lr,
 synsem:(loc:cat:(head:(mod_adj,
                         subj:[Subj],
                         da:(DA,
                             list_of_blocked_arguments)),
                   spr:Spr,
                   subcat:Subcat),
         nonloc:Nonloc,
         lex:Lex,
         trace:Trace),
 zu:Zu,
 dtrs:[(word,
        synsem:(loc:cat:(head:(verb,
                               vform:bse,
                               subj:[Subj],
                               da:DA),
                         spr:Spr,
                         subcat:Subcat),
                nonloc:Nonloc,
                lex:Lex,
                trace:Trace),
        zu:Zu)]).

bse_part_derivation_lr ##
  Dtr
 **>
( bse_part_derivation_lr,
  dtrs:[Dtr]
  )
morphs
  X becomes (X,d).

